@@ -1,5 +1,6 @@
+// -*- Mode: C++; indent-tabs-mode: nil; tab-width: 2 -*-
 /*
- * Copyright 2010 Inalogic® Inc.
+ * Copyright 2010-2011 Inalogic® Inc.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License, as
@@ -31,106 +32,98 @@ namespace nux
 {
   NUX_IMPLEMENT_OBJECT_TYPE(ScrollView);
 
-  ScrollView::ScrollView (NUX_FILE_LINE_DECL)
-    :   View (NUX_FILE_LINE_PARAM)
+  ScrollView::ScrollView(NUX_FILE_LINE_DECL)
+    : View(NUX_FILE_LINE_PARAM)
+    , m_MouseWheelScrollSize(32)
+      // TODO: these should really be Rects.
+    , m_ViewContentX(0)
+    , m_ViewContentY(0)
+    , m_ViewContentWidth(0)
+    , m_ViewContentHeight(0)
+    , m_ViewX(0)
+    , m_ViewY(0)
+    , m_ViewWidth(0)
+    , m_ViewHeight(0)
+    , m_TextureIndex(0)
+    , m_ReformatTexture(true)
+    , m_horizontal_scrollbar_enable(true)
+    , m_vertical_scrollbar_enable(true)
+    , m_top_border(0)
+    , m_border(0)
+    , _delta_x(0)
+    , _delta_y(0)
+    , m_bSizeMatchContent(false)
+    , m_ViewContentLeftMargin(0)
+    , m_ViewContentRightMargin(0)
+    , m_ViewContentTopMargin(0)
+    , m_ViewContentBottomMargin(0)
   {
-    m_vertical_scrollbar_enable     = true;
-    m_horizontal_scrollbar_enable   = true;
-    m_bSizeMatchContent             = false;
-    m_TextureIndex                  = 0;
-    m_ReformatTexture               = true;
-
-    _delta_x                        = 0;
-    _delta_y                        = 0;
 
     //GetPainter().CreateBackgroundTexture(m_BackgroundTexture);
-    hscrollbar = new HScrollBar (NUX_TRACKER_LOCATION);
-    vscrollbar = new VScrollBar (NUX_TRACKER_LOCATION);
+    _hscrollbar = new HScrollBar (NUX_TRACKER_LOCATION);
+    _vscrollbar = new VScrollBar (NUX_TRACKER_LOCATION);
+    // _hscrollbar and _vscrollbar have to be parented so they are correctly
+    // rendered and so that GetRootGeometry/GetAbsoluteGeometry returns the
+    // correct Geometry. This is necessary in embedded mode.
+    _hscrollbar->SetParentObject (this);
+    _vscrollbar->SetParentObject (this);
+
+    _hscrollbar->SetReconfigureParentLayoutOnGeometryChange(false);
+    _vscrollbar->SetReconfigureParentLayoutOnGeometryChange(false);
 
     SetMinimumSize (30, 30);
     SetGeometry (Geometry (0, 0, 400, 200) );
 
-    hscrollbar->OnScrollLeft.connect ( sigc::mem_fun (this, &ScrollView::ScrollLeft) );
-    hscrollbar->OnScrollRight.connect ( sigc::mem_fun (this, &ScrollView::ScrollRight) );
-    vscrollbar->OnScrollUp.connect ( sigc::mem_fun (this, &ScrollView::ScrollUp) );
-    vscrollbar->OnScrollDown.connect ( sigc::mem_fun (this, &ScrollView::ScrollDown) );
+    _hscrollbar->OnScrollLeft.connect ( sigc::mem_fun (this, &ScrollView::ScrollLeft) );
+    _hscrollbar->OnScrollRight.connect ( sigc::mem_fun (this, &ScrollView::ScrollRight) );
+    _vscrollbar->OnScrollUp.connect ( sigc::mem_fun (this, &ScrollView::ScrollUp) );
+    _vscrollbar->OnScrollDown.connect ( sigc::mem_fun (this, &ScrollView::ScrollDown) );
 
-    // hscrollbar and vscrollbar have to be parented so they are correctly
-    // rendered and so that GetRootGeometry/GetAbsoluteGeometry returns the
-    // correct Geometry. This is necessary in embedded mode.
-    hscrollbar->SetParentObject (this);
-    vscrollbar->SetParentObject (this);
+    mouse_wheel.connect(sigc::mem_fun(this, &ScrollView::RecvMouseWheel));
+    _vscrollbar->mouse_wheel.connect(sigc::mem_fun(this, &ScrollView::RecvMouseWheel));
 
-    setTopBorder (4);
-    setBorder (4);
-
-    m_MouseWheelScrollSize = 32;
-
-    m_ViewContentLeftMargin      = 0;
-    m_ViewContentRightMargin     = 0;
-    m_ViewContentTopMargin       = 0;
-    m_ViewContentBottomMargin    = 0;
     FormatContent();
 
-    ChildFocusChanged.connect (sigc::mem_fun (this, &ScrollView::OnChildFocusChanged));
+    //FIXME disabling until we have better API for this
+    //ChildFocusChanged.connect (sigc::mem_fun (this, &ScrollView::OnChildFocusChanged));
 
+    SetAcceptMouseWheelEvent(true);
   }
 
-  void
-  ScrollView::SetVScrollBar (VScrollBar* newVScrollBar)
+  // customization for Unity
+  void ScrollView::SetVScrollBar (VScrollBar* newVScrollBar)
   {
-    if (vscrollbar)
+    if(_vscrollbar)
     {
-      // disconnect old vscrollbar
-      vscrollbar->OnScrollUp.connect (sigc::mem_fun (this,
+      // disconnect old _vscrollbar
+      _vscrollbar->OnScrollUp.connect (sigc::mem_fun (this,
                                                      &ScrollView::ScrollUp));
-      vscrollbar->OnScrollDown.connect (sigc::mem_fun (this,
+      _vscrollbar->OnScrollDown.connect (sigc::mem_fun (this,
                                                        &ScrollView::ScrollDown));
-      if (vscrollbar->OwnsTheReference ())
-      {
-        vscrollbar->UnReference ();
-      }
-      else
-      {
-        vscrollbar->Dispose ();
-      }
+      _vscrollbar->UnReference ();
     }
 
-    vscrollbar = newVScrollBar;
+    _vscrollbar = newVScrollBar;
 
-    vscrollbar->Reference ();
-    vscrollbar->SetParentObject (this);
+    _vscrollbar->Reference ();
+    _vscrollbar->SetParentObject (this);
+    _vscrollbar->SetReconfigureParentLayoutOnGeometryChange(false);
 
-    // connect new vscrollbar
-    vscrollbar->OnScrollUp.connect (sigc::mem_fun (this,
+    // connect new _vscrollbar
+    _vscrollbar->OnScrollUp.connect (sigc::mem_fun (this,
                                                    &ScrollView::ScrollUp));
-    vscrollbar->OnScrollDown.connect (sigc::mem_fun (this,
+    _vscrollbar->OnScrollDown.connect (sigc::mem_fun (this,
                                                      &ScrollView::ScrollDown));
   }
 
   ScrollView::~ScrollView()
   {
     // Delete all the interface object: This is a problem... The widget should be destroy by there associated parameters
-    if (hscrollbar->OwnsTheReference ())
-    {
-      hscrollbar->UnReference ();
-    }
-    else
-    {
-      hscrollbar->Dispose ();
-    }
-
-    if (vscrollbar->OwnsTheReference ())
-    {
-      vscrollbar->UnReference ();
-    }
-    else
-    {
-      vscrollbar->Dispose ();
-    }
+    _hscrollbar->UnReference ();
+    _vscrollbar->UnReference ();
   }
 
-  void ScrollView::OnChildFocusChanged (Area *parent, Area *child)
+  void ScrollView::OnChildFocusChanged (/*Area *parent,*/ Area *child)
   {
     if (child->IsView ())
     {
@@ -142,16 +135,16 @@ namespace nux
     }
     if (child->IsLayout ())
       return;
-    
+
     int child_y = child->GetGeometry ().y - GetGeometry ().y;
     int child_y_diff = child_y - abs (_delta_y);
 
-    
+
     if (child_y_diff + child->GetGeometry ().height < GetGeometry ().height && child_y_diff >= 0)
     {
       return;
     }
-    
+
     if (child_y_diff < 0)
     {
       ScrollUp (1, abs (child_y_diff));
@@ -162,13 +155,12 @@ namespace nux
 
       // always keeps the top of a view on the screen
       size += (child->GetGeometry ().height, GetGeometry ().height) ? child->GetGeometry ().height : GetGeometry ().height;
-      
+
       ScrollDown (1, size);
     }
 
   }
 
-
   long ScrollView::ProcessEvent (Event &event, long TraverseInfo, long ProcessEventInfo)
   {
     long ret = TraverseInfo;
@@ -183,7 +175,7 @@ namespace nux
       }
     }
 
-    if (event.e_event == NUX_MOUSEWHEEL)
+    if (event.e_event == NUX_MOUSE_WHEEL)
     {
       // nux can't tell the difference between horizontal and vertical mouse wheel events
       // so we are only going to support vertical
@@ -198,14 +190,13 @@ namespace nux
     }
 
     if (m_vertical_scrollbar_enable)
-      ret = vscrollbar->ProcessEvent (event, ret, ProcEvInfo);
+      ret = _vscrollbar->ProcessEvent (event, ret, ProcEvInfo);
 
     if (m_horizontal_scrollbar_enable)
-      ret = hscrollbar->ProcessEvent (event, ret, ProcEvInfo);
+      ret = _hscrollbar->ProcessEvent (event, ret, ProcEvInfo);
 
     // The child layout get the Mouse down button only if the MouseDown happened inside the client view Area
     Geometry viewGeometry = Geometry (m_ViewX, m_ViewY, m_ViewWidth, m_ViewHeight);
-    bool traverse = true;
 
     if ((event.e_event == NUX_MOUSE_PRESSED) ||
      (event.e_event == NUX_MOUSE_RELEASED) ||
@@ -214,7 +205,6 @@ namespace nux
       if (!viewGeometry.IsPointInside (event.e_x - event.e_x_root, event.e_y - event.e_y_root) )
       {
         ProcEvInfo = eDoNotProcess;
-        traverse = false;
       }
     }
 
@@ -226,12 +216,43 @@ namespace nux
       ret = m_CompositionLayout->ProcessEvent (mod_event, ret, ProcEvInfo);
     }
 
-
-
     ret = PostProcessEvent2 (event, ret, 0);
     return ret;
   }
 
+  Area* ScrollView::FindAreaUnderMouse(const Point& mouse_position, NuxEventType event_type)
+  {
+    // Test if the mouse is inside the ScrollView.
+    // The last parameter of TestMousePointerInclusion is a boolean used to test if the case
+    // of mouse wheel events. If that boolean value is true, then TestMousePointerInclusion
+    // returns true only if the mouse pointer is over this area and the the area accepts
+    // mouse wheel events (see Area::SetAcceptMouseWheelEvent)
+    bool mouse_inside = TestMousePointerInclusionFilterMouseWheel(mouse_position, event_type);
+
+    if(mouse_inside == false)
+    {
+      // The mouse pointer is not over this Area. return NULL.
+      return NULL;
+    }
+
+    Area* found_area;
+
+    // Recursively go over the ui element that are managed by this ScrollView and look
+    // for the area that is below the mouse.
+
+    // Test the vertical scrollbar
+    found_area = _vscrollbar->FindAreaUnderMouse(mouse_position, event_type);
+    NUX_RETURN_VALUE_IF_NOTNULL(found_area, found_area);
+
+    // Test the horizontal scrollbar
+    found_area = _hscrollbar->FindAreaUnderMouse(mouse_position, event_type);
+    NUX_RETURN_VALUE_IF_NOTNULL(found_area, found_area);
+
+    // If the code gets here, it means that no area has been found yet.
+    // Test the layout of the ScrollView
+    return View::FindAreaUnderMouse(mouse_position, event_type);
+  }
+
   bool ScrollView::SetLayout (Layout *layout)
   {
     if(View::SetLayout(layout) == false)
@@ -251,18 +272,18 @@ namespace nux
     Geometry base = GetGeometry ();
 
     if (m_CompositionLayout)
-      m_CompositionLayout->NeedRedraw();
+      m_CompositionLayout->QueueDraw();
 
     GetPainter().PaintBackground (GfxContext, base);
 
     if (m_vertical_scrollbar_enable)
     {
-      vscrollbar->NeedRedraw();
+      _vscrollbar->QueueDraw();
     }
 
     if (m_horizontal_scrollbar_enable)
     {
-      hscrollbar->NeedRedraw();
+      _hscrollbar->QueueDraw();
     }
 
     GfxContext.PopClippingRectangle();
@@ -270,6 +291,9 @@ namespace nux
 
   void ScrollView::DrawContent (GraphicsEngine &GfxContext, bool force_draw)
   {
+    if (IsFullRedraw())
+      GetPainter().PushBackgroundStack();
+      
     GfxContext.PushClippingRectangle (GetGeometry ());
 
     GfxContext.PushClippingRectangle (Rect (m_ViewX, m_ViewY, m_ViewWidth, m_ViewHeight));
@@ -289,15 +313,18 @@ namespace nux
 
     if (m_vertical_scrollbar_enable)
     {
-      vscrollbar->ProcessDraw (GfxContext, force_draw);
+      _vscrollbar->ProcessDraw (GfxContext, force_draw);
     }
 
     if (m_horizontal_scrollbar_enable)
     {
-      hscrollbar->ProcessDraw (GfxContext, force_draw);
+      _hscrollbar->ProcessDraw (GfxContext, force_draw);
     }
 
     GfxContext.PopClippingRectangle();
+
+    if (IsFullRedraw())
+      GetPainter().PopBackgroundStack();
   }
 
   void ScrollView::PostDraw (GraphicsEngine &GfxContext, bool force_draw)
@@ -325,25 +352,6 @@ namespace nux
 ///////////////////////
 // Internal function //
 ///////////////////////
-  void ScrollView::setBorder (int border)
-  {
-    m_border = border;
-  }
-
-  void ScrollView::setTopBorder (int top_border)
-  {
-    m_top_border = top_border;
-  }
-
-  int ScrollView::getBorder() const
-  {
-    return m_border;
-  }
-
-  int ScrollView::getTopBorder() const
-  {
-    return m_top_border;
-  }
 
   void ScrollView::SetGeometry (const Geometry &geo)
   {
@@ -364,11 +372,11 @@ namespace nux
     // Give the managed layout the same size and position as the Control.
 
     Geometry geo = GetGeometry();
-    int ScrollBarWidth = vscrollbar->GetBaseWidth();
-    int ScrollBarHeight = hscrollbar->GetBaseHeight();
+    int ScrollBarWidth = _vscrollbar->GetBaseWidth();
+    int ScrollBarHeight = _hscrollbar->GetBaseHeight();
 
-    nuxAssertMsg (ScrollBarWidth > 0, TEXT ("[ScrollView::PreLayoutManagement] Invalid scrollbar width.") );
-    nuxAssertMsg (ScrollBarHeight > 0, TEXT ("[ScrollView::PreLayoutManagement] Invalid scrollbar height.") );
+    nuxAssertMsg (ScrollBarWidth > 0, TEXT ("[ScrollView::PreLayoutManagement] Invalid scrollbar width: %d"), ScrollBarWidth);
+    nuxAssertMsg (ScrollBarHeight > 0, TEXT ("[ScrollView::PreLayoutManagement] Invalid scrollbar height: %d"), ScrollBarHeight);
 
     m_ViewX = GetBaseX() + m_border + m_ViewContentLeftMargin;
     m_ViewY = GetBaseY() + m_top_border + m_ViewContentTopMargin;
@@ -378,14 +386,14 @@ namespace nux
     else
       m_ViewWidth = GetBaseWidth() - ScrollBarWidth - 2 * m_border - m_ViewContentRightMargin - m_ViewContentLeftMargin;
 
-    nuxAssertMsg (m_ViewWidth > 0, TEXT ("[ScrollView::PreLayoutManagement] Invalid view width.") );
+    nuxAssertMsg (m_ViewWidth > 0, TEXT ("[ScrollView::PreLayoutManagement] Invalid view width: %d"), m_ViewWidth);
 
     if (m_horizontal_scrollbar_enable == false)
       m_ViewHeight = GetBaseHeight() - m_top_border - m_border - m_ViewContentBottomMargin - m_ViewContentTopMargin;
     else
       m_ViewHeight = GetBaseHeight() - ScrollBarHeight - m_top_border - m_border - m_ViewContentBottomMargin - m_ViewContentTopMargin;
 
-    nuxAssertMsg (m_ViewWidth > 0, TEXT ("[ScrollView::PreLayoutManagement] Invalid view height.") );
+    nuxAssertMsg (m_ViewHeight > 0, TEXT ("[ScrollView::PreLayoutManagement] Invalid view height: %d"), m_ViewHeight);
 
     if (m_ViewX + _delta_x +  m_ViewContentWidth < m_ViewX + m_ViewWidth)
     {
@@ -425,22 +433,22 @@ namespace nux
       if (m_vertical_scrollbar_enable == false)
       {
         // If there is no vertical scrollbar, take all the width available.
-        hscrollbar->SetBaseWidth (GetBaseWidth() - 2 * m_border);
+        _hscrollbar->SetBaseWidth (GetBaseWidth() - 2 * m_border);
       }
       else
-        hscrollbar->SetBaseWidth (GetBaseWidth() - ScrollBarWidth - 2 * m_border);
+        _hscrollbar->SetBaseWidth (GetBaseWidth() - ScrollBarWidth - 2 * m_border);
 
-      hscrollbar->SetBaseX (geo.x + m_border);
-      hscrollbar->SetBaseY (geo.y + geo.GetHeight() - hscrollbar->GetBaseHeight() - m_border);
-      hscrollbar->ComputeChildLayout();
+      _hscrollbar->SetBaseX (geo.x + m_border);
+      _hscrollbar->SetBaseY (geo.y + geo.GetHeight() - _hscrollbar->GetBaseHeight() - m_border);
+      _hscrollbar->ComputeChildLayout();
     }
     else
     {
       // The horizontal scrollbar won't be visible but give it a proper size anyway.
-      hscrollbar->SetBaseWidth (GetBaseWidth() - ScrollBarWidth - 2 * m_border);
-      hscrollbar->SetBaseX (geo.x + m_border);
-      hscrollbar->SetBaseY (geo.y + geo.GetHeight() - hscrollbar->GetBaseHeight() - m_border);
-      hscrollbar->ComputeChildLayout();
+      _hscrollbar->SetBaseWidth (GetBaseWidth() - ScrollBarWidth - 2 * m_border);
+      _hscrollbar->SetBaseX (geo.x + m_border);
+      _hscrollbar->SetBaseY (geo.y + geo.GetHeight() - _hscrollbar->GetBaseHeight() - m_border);
+      _hscrollbar->ComputeChildLayout();
     }
 
 
@@ -450,22 +458,22 @@ namespace nux
       if (m_horizontal_scrollbar_enable == false)
       {
         // If there is no horizontal scrollbar, take all the width available.
-        vscrollbar->SetBaseHeight (GetBaseHeight() - m_top_border - m_border);
+        _vscrollbar->SetBaseHeight (GetBaseHeight() - m_top_border - m_border);
       }
       else
-        vscrollbar->SetBaseHeight (GetBaseHeight() - ScrollBarHeight - m_top_border - m_border);
+        _vscrollbar->SetBaseHeight (GetBaseHeight() - ScrollBarHeight - m_top_border - m_border);
 
-      vscrollbar->SetBaseX (geo.x + geo.GetWidth() - ScrollBarWidth - m_border);
-      vscrollbar->SetBaseY (geo.y + m_top_border);
-      vscrollbar->ComputeChildLayout();
+      _vscrollbar->SetBaseX (geo.x + geo.GetWidth() - ScrollBarWidth - m_border);
+      _vscrollbar->SetBaseY (geo.y + m_top_border);
+      _vscrollbar->ComputeChildLayout();
     }
     else
     {
       // The vertical scrollbar won't be visible but give it a proper size anyway.
-      vscrollbar->SetBaseHeight (GetBaseHeight() - ScrollBarHeight - m_top_border - m_border);
-      vscrollbar->SetBaseX (geo.x + geo.GetWidth() - ScrollBarWidth - m_border);
-      vscrollbar->SetBaseY (geo.y + m_top_border);
-      vscrollbar->ComputeChildLayout();
+      _vscrollbar->SetBaseHeight (GetBaseHeight() - ScrollBarHeight - m_top_border - m_border);
+      _vscrollbar->SetBaseX (geo.x + geo.GetWidth() - ScrollBarWidth - m_border);
+      _vscrollbar->SetBaseY (geo.y + m_top_border);
+      _vscrollbar->ComputeChildLayout();
     }
   }
 
@@ -486,12 +494,12 @@ namespace nux
     }
 
     if (m_horizontal_scrollbar_enable)
-      ScrollBarHeight = hscrollbar->GetBaseHeight();
+      ScrollBarHeight = _hscrollbar->GetBaseHeight();
 
     if (m_vertical_scrollbar_enable)
-      ScrollBarWidth = vscrollbar->GetBaseWidth();
+      ScrollBarWidth = _vscrollbar->GetBaseWidth();
 
-    hscrollbar->SetContainerSize (GetBaseX() + m_border + m_ViewContentLeftMargin,
+    _hscrollbar->SetContainerSize (GetBaseX() + m_border + m_ViewContentLeftMargin,
                                   GetBaseY() + m_top_border + m_ViewContentTopMargin,
                                   GetBaseWidth() - ScrollBarWidth - 2 * m_border - m_ViewContentRightMargin - m_ViewContentLeftMargin,
                                   GetBaseHeight() - ScrollBarHeight - m_top_border - m_border - m_ViewContentBottomMargin - m_ViewContentTopMargin);
@@ -500,25 +508,25 @@ namespace nux
     {
       if (m_CompositionLayout)
       {
-        hscrollbar->SetContentSize (m_CompositionLayout->GetBaseX(), m_CompositionLayout->GetBaseY(),
+        _hscrollbar->SetContentSize (m_CompositionLayout->GetBaseX(), m_CompositionLayout->GetBaseY(),
                                     m_CompositionLayout->GetBaseWidth(), m_CompositionLayout->GetBaseHeight() );
       }
       else
       {
-        hscrollbar->SetContentSize (GetBaseX() + m_border + m_ViewContentLeftMargin,
+        _hscrollbar->SetContentSize (GetBaseX() + m_border + m_ViewContentLeftMargin,
                                     GetBaseY() + m_top_border + m_ViewContentTopMargin, 0, 0);
       }
 
-      hscrollbar->SetContentOffset (_delta_x, _delta_y);
+      _hscrollbar->SetContentOffset (_delta_x, _delta_y);
     }
     else
     {
-      hscrollbar->SetContentSize (GetBaseX() + m_border + m_ViewContentLeftMargin,
+      _hscrollbar->SetContentSize (GetBaseX() + m_border + m_ViewContentLeftMargin,
                                   GetBaseY() + m_top_border + m_ViewContentTopMargin, 0, 0);
-      hscrollbar->SetContentOffset (0, 0);
+      _hscrollbar->SetContentOffset (0, 0);
     }
 
-    vscrollbar->SetContainerSize (GetBaseX() + m_border + m_ViewContentLeftMargin,
+    _vscrollbar->SetContainerSize (GetBaseX() + m_border + m_ViewContentLeftMargin,
                                   GetBaseY() + m_top_border + m_ViewContentTopMargin,
                                   GetBaseWidth() - ScrollBarWidth - 2 * m_border - m_ViewContentRightMargin - m_ViewContentLeftMargin,
                                   GetBaseHeight() - ScrollBarHeight - m_top_border - m_border - m_ViewContentBottomMargin - m_ViewContentTopMargin);
@@ -527,22 +535,22 @@ namespace nux
     {
       if (m_CompositionLayout)
       {
-        vscrollbar->SetContentSize (m_CompositionLayout->GetBaseX(), m_CompositionLayout->GetBaseY(),
+        _vscrollbar->SetContentSize (m_CompositionLayout->GetBaseX(), m_CompositionLayout->GetBaseY(),
                                     m_CompositionLayout->GetBaseWidth(), m_CompositionLayout->GetBaseHeight() );
       }
       else
       {
-        vscrollbar->SetContentSize (GetBaseX() + m_border + m_ViewContentLeftMargin,
+        _vscrollbar->SetContentSize (GetBaseX() + m_border + m_ViewContentLeftMargin,
                                     GetBaseY() + m_top_border + m_ViewContentTopMargin, 0, 0);
       }
 
-      vscrollbar->SetContentOffset (_delta_x, _delta_y);
+      _vscrollbar->SetContentOffset (_delta_x, _delta_y);
     }
     else
     {
-      vscrollbar->SetContentSize (GetBaseX() + m_border + m_ViewContentLeftMargin,
+      _vscrollbar->SetContentSize (GetBaseX() + m_border + m_ViewContentLeftMargin,
                                   GetBaseY() + m_top_border + m_ViewContentTopMargin, 0, 0);
-      vscrollbar->SetContentOffset (0, 0);
+      _vscrollbar->SetContentOffset (0, 0);
     }
 
     // I may not be necessary to call this function here since ComputePosition2 was called on ComputePosition2
@@ -561,10 +569,10 @@ namespace nux
     int ScrollbarHeight = 0;
 
     if (m_horizontal_scrollbar_enable)
-      ScrollbarHeight = hscrollbar->GetBaseHeight();
+      ScrollbarHeight = _hscrollbar->GetBaseHeight();
 
     if (m_vertical_scrollbar_enable)
-      ScrollbarWidth = vscrollbar->GetBaseWidth();
+      ScrollbarWidth = _vscrollbar->GetBaseWidth();
 
     // We want the controller to match the size of the content as defined in:
     //      m_ViewContentX
@@ -607,48 +615,48 @@ namespace nux
     if (m_horizontal_scrollbar_enable)
     {
       if (m_vertical_scrollbar_enable == false)
-        hscrollbar->SetBaseWidth (GetBaseWidth() - 2 * m_border);
+        _hscrollbar->SetBaseWidth (GetBaseWidth() - 2 * m_border);
       else
-        hscrollbar->SetBaseWidth (GetBaseWidth() - ScrollbarWidth - 2 * m_border);
+        _hscrollbar->SetBaseWidth (GetBaseWidth() - ScrollbarWidth - 2 * m_border);
 
-      hscrollbar->SetBaseX (geo.x + m_border);
-      hscrollbar->SetBaseY (geo.y + geo.GetHeight() - hscrollbar->GetBaseHeight() - m_border);
-      hscrollbar->ComputeChildLayout();
+      _hscrollbar->SetBaseX (geo.x + m_border);
+      _hscrollbar->SetBaseY (geo.y + geo.GetHeight() - _hscrollbar->GetBaseHeight() - m_border);
+      _hscrollbar->ComputeChildLayout();
 
       //---
-      hscrollbar->SetContainerSize (GetBaseX() + m_border + m_ViewContentLeftMargin,
+      _hscrollbar->SetContainerSize (GetBaseX() + m_border + m_ViewContentLeftMargin,
                                     GetBaseY() + m_top_border + m_ViewContentTopMargin,
                                     GetBaseWidth() - ScrollbarWidth - 2 * m_border - m_ViewContentRightMargin - m_ViewContentLeftMargin,
                                     GetBaseHeight() - ScrollbarHeight - m_top_border - m_border - m_ViewContentBottomMargin - m_ViewContentTopMargin);
 
       if (m_CompositionLayout)
       {
-        hscrollbar->SetContentSize (m_CompositionLayout->GetBaseX(), m_CompositionLayout->GetBaseY(),
+        _hscrollbar->SetContentSize (m_CompositionLayout->GetBaseX(), m_CompositionLayout->GetBaseY(),
                                     m_CompositionLayout->GetBaseWidth(), m_CompositionLayout->GetBaseHeight() );
       }
       else
       {
-        hscrollbar->SetContentSize (GetBaseX() + m_border + m_ViewContentLeftMargin,
+        _hscrollbar->SetContentSize (GetBaseX() + m_border + m_ViewContentLeftMargin,
                                     GetBaseY() + m_top_border + m_ViewContentTopMargin, 0, 0);
       }
 
-      hscrollbar->SetContentOffset (_delta_x, _delta_y);
+      _hscrollbar->SetContentOffset (_delta_x, _delta_y);
     }
     else
     {
-      hscrollbar->SetBaseWidth (GetBaseWidth() - ScrollbarWidth - 2 * m_border);
-      hscrollbar->SetBaseX (geo.x + m_border);
-      hscrollbar->SetBaseY (geo.y + geo.GetHeight() - hscrollbar->GetBaseHeight() - m_border);
-      hscrollbar->ComputeChildLayout();
+      _hscrollbar->SetBaseWidth (GetBaseWidth() - ScrollbarWidth - 2 * m_border);
+      _hscrollbar->SetBaseX (geo.x + m_border);
+      _hscrollbar->SetBaseY (geo.y + geo.GetHeight() - _hscrollbar->GetBaseHeight() - m_border);
+      _hscrollbar->ComputeChildLayout();
 
       //---
-      hscrollbar->SetContainerSize (GetBaseX() + m_border + m_ViewContentLeftMargin,
+      _hscrollbar->SetContainerSize (GetBaseX() + m_border + m_ViewContentLeftMargin,
                                     GetBaseY() + m_top_border + m_ViewContentTopMargin,
                                     GetBaseWidth() - ScrollbarWidth - 2 * m_border - m_ViewContentRightMargin - m_ViewContentLeftMargin,
                                     GetBaseHeight() - ScrollbarHeight - m_top_border - m_border - m_ViewContentBottomMargin - m_ViewContentTopMargin);
-      hscrollbar->SetContentSize (GetBaseX() + m_border + m_ViewContentLeftMargin,
+      _hscrollbar->SetContentSize (GetBaseX() + m_border + m_ViewContentLeftMargin,
                                   GetBaseY() + m_top_border + m_ViewContentTopMargin, 0, 0);
-      hscrollbar->SetContentOffset (0, 0);
+      _hscrollbar->SetContentOffset (0, 0);
     }
 
 
@@ -656,48 +664,48 @@ namespace nux
     if (m_vertical_scrollbar_enable)
     {
       if (m_horizontal_scrollbar_enable == false)
-        vscrollbar->SetBaseHeight (GetBaseHeight() - m_top_border - m_border);
+        _vscrollbar->SetBaseHeight (GetBaseHeight() - m_top_border - m_border);
       else
-        vscrollbar->SetBaseHeight (GetBaseHeight() - ScrollbarHeight - m_top_border - m_border);
+        _vscrollbar->SetBaseHeight (GetBaseHeight() - ScrollbarHeight - m_top_border - m_border);
 
-      vscrollbar->SetBaseX (geo.x + geo.GetWidth() - ScrollbarWidth - m_border);
-      vscrollbar->SetBaseY (geo.y + m_top_border);
-      vscrollbar->ComputeChildLayout();
+      _vscrollbar->SetBaseX (geo.x + geo.GetWidth() - ScrollbarWidth - m_border);
+      _vscrollbar->SetBaseY (geo.y + m_top_border);
+      _vscrollbar->ComputeChildLayout();
 
       //---
-      vscrollbar->SetContainerSize (GetBaseX() + m_border + m_ViewContentLeftMargin,
+      _vscrollbar->SetContainerSize (GetBaseX() + m_border + m_ViewContentLeftMargin,
                                     GetBaseY() + m_top_border + m_ViewContentTopMargin,
                                     GetBaseWidth() - ScrollbarWidth - 2 * m_border - m_ViewContentRightMargin - m_ViewContentLeftMargin,
                                     GetBaseHeight() - ScrollbarHeight - m_top_border - m_border - m_ViewContentBottomMargin - m_ViewContentTopMargin);
 
       if (m_CompositionLayout)
       {
-        vscrollbar->SetContentSize (m_CompositionLayout->GetBaseX(), m_CompositionLayout->GetBaseY(),
+        _vscrollbar->SetContentSize (m_CompositionLayout->GetBaseX(), m_CompositionLayout->GetBaseY(),
                                     m_CompositionLayout->GetBaseWidth(), m_CompositionLayout->GetBaseHeight() );
       }
       else
       {
-        vscrollbar->SetContentSize (GetBaseX() + m_border + m_ViewContentLeftMargin,
+        _vscrollbar->SetContentSize (GetBaseX() + m_border + m_ViewContentLeftMargin,
                                     GetBaseY() + m_top_border + m_ViewContentTopMargin, 0, 0);
       }
 
-      vscrollbar->SetContentOffset (_delta_x, _delta_y);
+      _vscrollbar->SetContentOffset (_delta_x, _delta_y);
     }
     else
     {
-      vscrollbar->SetBaseHeight (GetBaseHeight() - ScrollbarHeight - m_top_border - m_border);
-      vscrollbar->SetBaseX (geo.x + geo.GetWidth() - ScrollbarWidth - m_border);
-      vscrollbar->SetBaseY (geo.y + m_top_border);
-      vscrollbar->ComputeChildLayout();
+      _vscrollbar->SetBaseHeight (GetBaseHeight() - ScrollbarHeight - m_top_border - m_border);
+      _vscrollbar->SetBaseX (geo.x + geo.GetWidth() - ScrollbarWidth - m_border);
+      _vscrollbar->SetBaseY (geo.y + m_top_border);
+      _vscrollbar->ComputeChildLayout();
 
       //---
-      vscrollbar->SetContainerSize (GetBaseX() + m_border + m_ViewContentLeftMargin,
+      _vscrollbar->SetContainerSize (GetBaseX() + m_border + m_ViewContentLeftMargin,
                                     GetBaseY() + m_top_border + m_ViewContentTopMargin,
                                     GetBaseWidth() - ScrollbarWidth - 2 * m_border - m_ViewContentRightMargin - m_ViewContentLeftMargin,
                                     GetBaseHeight() - ScrollbarHeight - m_top_border - m_border - m_ViewContentBottomMargin - m_ViewContentTopMargin);
-      vscrollbar->SetContentSize (GetBaseX() + m_border + m_ViewContentLeftMargin,
+      _vscrollbar->SetContentSize (GetBaseX() + m_border + m_ViewContentLeftMargin,
                                   GetBaseY() + m_top_border + m_ViewContentTopMargin, 0, 0);
-      vscrollbar->SetContentOffset (0, 0);
+      _vscrollbar->SetContentOffset (0, 0);
     }
 
     if (m_CompositionLayout)
@@ -718,21 +726,8 @@ namespace nux
     int w = 0;
     int h = 0;
 
-    w = vscrollbar->GetBaseWidth();
-    h = hscrollbar->GetBaseHeight();
-
-//    m_ViewX = GetX() + m_border;
-//    m_ViewY = GetY() + m_top_border;
-//
-//    if(m_vertical_scrollbar_enable == false)
-//        m_ViewWidth = GetWidth() - 2*m_border - m_ViewContentRightMargin;
-//    else
-//        m_ViewWidth = GetWidth() - w - 2*m_border - m_ViewContentRightMargin;
-//
-//    if(m_horizontal_scrollbar_enable == false)
-//        m_ViewHeight = GetBaseHeight() - m_top_border - m_border - m_ViewContentBottomMargin;
-//    else
-//        m_ViewHeight = GetBaseHeight() - h - m_top_border - m_border - m_ViewContentBottomMargin;
+    w = _vscrollbar->GetBaseWidth();
+    h = _hscrollbar->GetBaseHeight();
 
     m_ViewX = GetBaseX() + m_border + m_ViewContentLeftMargin;
     m_ViewY = GetBaseY() + m_top_border + m_ViewContentTopMargin;
@@ -768,26 +763,26 @@ namespace nux
     if (m_horizontal_scrollbar_enable)
     {
       if (m_vertical_scrollbar_enable == false)
-        hscrollbar->SetBaseWidth (GetBaseWidth() - 2 * m_border);
+        _hscrollbar->SetBaseWidth (GetBaseWidth() - 2 * m_border);
       else
-        hscrollbar->SetBaseWidth (GetBaseWidth() - w - 2 * m_border);
+        _hscrollbar->SetBaseWidth (GetBaseWidth() - w - 2 * m_border);
 
-      hscrollbar->SetBaseX (geo.x + m_border);
-      hscrollbar->SetBaseY (geo.y + geo.GetHeight() - hscrollbar->GetBaseHeight() - m_border);
-      hscrollbar->ComputeChildLayout();
+      _hscrollbar->SetBaseX (geo.x + m_border);
+      _hscrollbar->SetBaseY (geo.y + geo.GetHeight() - _hscrollbar->GetBaseHeight() - m_border);
+      _hscrollbar->ComputeChildLayout();
     }
 
     // Vertical scrollbar Geometry
     if (m_vertical_scrollbar_enable)
     {
       if (m_horizontal_scrollbar_enable == false)
-        vscrollbar->SetBaseHeight (GetBaseHeight() - m_top_border - m_border);
+        _vscrollbar->SetBaseHeight (GetBaseHeight() - m_top_border - m_border);
       else
-        vscrollbar->SetBaseHeight (GetBaseHeight() - h - m_top_border - m_border);
+        _vscrollbar->SetBaseHeight (GetBaseHeight() - h - m_top_border - m_border);
 
-      vscrollbar->SetBaseX (geo.x + geo.GetWidth() - w - m_border);
-      vscrollbar->SetBaseY (geo.y + m_top_border);
-      vscrollbar->ComputeChildLayout();
+      _vscrollbar->SetBaseX (geo.x + geo.GetWidth() - w - m_border);
+      _vscrollbar->SetBaseY (geo.y + m_top_border);
+      _vscrollbar->ComputeChildLayout();
     }
 
     if (m_CompositionLayout)
@@ -801,17 +796,8 @@ namespace nux
       m_ViewContentY = m_ViewY;
     }
 
-    int hor_scrollbar_height = 0;
-    int ver_scrollbar_width = 0;
-
-    if (m_horizontal_scrollbar_enable == true)
-      hor_scrollbar_height = hscrollbar->GetBaseHeight();
-
-    if (m_vertical_scrollbar_enable == true)
-      ver_scrollbar_width = vscrollbar->GetBaseWidth();
-
-    vscrollbar->SetContentOffset (_delta_x, _delta_y);
-    hscrollbar->SetContentOffset (_delta_x, _delta_y);
+    _vscrollbar->SetContentOffset (_delta_x, _delta_y);
+    _hscrollbar->SetContentOffset (_delta_x, _delta_y);
 
     if (m_CompositionLayout)
     {
@@ -835,11 +821,11 @@ namespace nux
 
     if (m_CompositionLayout)
     {
-      hscrollbar->SetContentOffset (_delta_x, _delta_y);
-      hscrollbar->NeedRedraw();
+      _hscrollbar->SetContentOffset (_delta_x, _delta_y);
+      _hscrollbar->QueueDraw();
     }
 
-    NeedRedraw();
+    QueueDraw();
   }
 
   void ScrollView::ScrollRight (float stepx, int mousedx)
@@ -857,11 +843,11 @@ namespace nux
 
     if (m_CompositionLayout)
     {
-      hscrollbar->SetContentOffset (_delta_x, _delta_y);
-      hscrollbar->NeedRedraw();
+      _hscrollbar->SetContentOffset (_delta_x, _delta_y);
+      _hscrollbar->QueueDraw();
     }
 
-    NeedRedraw();
+    QueueDraw();
   }
 
   void ScrollView::ScrollUp (float stepy, int mousedy)
@@ -879,11 +865,11 @@ namespace nux
 
     if (m_CompositionLayout)
     {
-      vscrollbar->SetContentOffset (_delta_x, _delta_y);
-      vscrollbar->NeedRedraw();
+      _vscrollbar->SetContentOffset (_delta_x, _delta_y);
+      _vscrollbar->QueueDraw();
     }
 
-    NeedRedraw();
+    QueueDraw();
   }
 
   void ScrollView::ScrollDown (float stepy, int mousedy)
@@ -901,11 +887,11 @@ namespace nux
 
     if (m_CompositionLayout)
     {
-      vscrollbar->SetContentOffset (_delta_x, _delta_y);
-      vscrollbar->NeedRedraw();
+      _vscrollbar->SetContentOffset (_delta_x, _delta_y);
+      _vscrollbar->QueueDraw();
     }
 
-    NeedRedraw();
+    QueueDraw();
   }
 
   void ScrollView::SetSizeMatchContent (bool b)
@@ -927,10 +913,10 @@ namespace nux
     if (m_CompositionLayout)
       m_CompositionLayout->Set2DTranslation (_delta_x, _delta_y, 0);
 
-    hscrollbar->SetContentOffset (_delta_x, _delta_y);
-    hscrollbar->NeedRedraw();
+    _hscrollbar->SetContentOffset (_delta_x, _delta_y);
+    _hscrollbar->QueueDraw();
 
-    NeedRedraw();
+    QueueDraw();
   }
 
   void ScrollView::ResetScrollToRight()
@@ -947,10 +933,10 @@ namespace nux
 
 
 
-    hscrollbar->SetContentOffset (_delta_x, _delta_y);
-    hscrollbar->NeedRedraw();
+    _hscrollbar->SetContentOffset (_delta_x, _delta_y);
+    _hscrollbar->QueueDraw();
 
-    NeedRedraw();
+    QueueDraw();
   }
 
   void ScrollView::ResetScrollToUp()
@@ -959,10 +945,10 @@ namespace nux
     if (m_CompositionLayout)
       m_CompositionLayout->Set2DTranslation (_delta_x, _delta_y, 0);
 
-    vscrollbar->SetContentOffset (_delta_x, _delta_y);
-    vscrollbar->NeedRedraw();
+    _vscrollbar->SetContentOffset (_delta_x, _delta_y);
+    _vscrollbar->QueueDraw();
 
-    NeedRedraw();
+    QueueDraw();
   }
 
   void ScrollView::ResetScrollToDown()
@@ -977,9 +963,29 @@ namespace nux
       _delta_y = 0;
     }
 
-    vscrollbar->SetContentOffset (_delta_x, _delta_y);
-    vscrollbar->NeedRedraw();
+    _vscrollbar->SetContentOffset (_delta_x, _delta_y);
+    _vscrollbar->QueueDraw();
 
-    NeedRedraw();
+    QueueDraw();
+  }
+
+  void ScrollView::RecvMouseWheel(int x, int y, int wheel_delta, long button_flags, unsigned long key_flags)
+  {
+    // nux can't tell the difference between horizontal and vertical mouse wheel events
+    // so we are only going to support vertical
+    if (wheel_delta < 0)
+    {
+      ScrollDown (abs (wheel_delta / NUX_MOUSEWHEEL_DELTA), m_MouseWheelScrollSize);
+    }
+    else
+    {
+      ScrollUp (abs (wheel_delta / NUX_MOUSEWHEEL_DELTA), m_MouseWheelScrollSize);
+    }
+    QueueDraw();
+  }
+
+  bool ScrollView::AcceptKeyNavFocus()
+  {
+    return false;
   }
 }
