@@ -28,7 +28,7 @@
 namespace nux
 {
 
-  static const t_s32 VERROR = 0;
+  static const int VERROR = 0;
   NUX_IMPLEMENT_OBJECT_TYPE (VLayout);
 
   VLayout::VLayout (NUX_FILE_LINE_DECL)
@@ -111,11 +111,11 @@ namespace nux
       @param offset_space     The space at the top of all elements.
       @param element_margin   The margin between elements.
   */
-  void VLayout::ComputeStacking (t_s32 remaining_height, t_s32 &offset_space, t_s32 &element_margin)
+  void VLayout::ComputeStacking (int remaining_height, int &offset_space, int &element_margin)
   {
-    t_s32 per_element_space = 0;
-    t_s32 total_used_space = 0;
-    t_s32 n_elements = 0;
+    int per_element_space = 0;
+    int total_used_space = 0;
+    int n_elements = 0;
 
     std::list<Area *>::iterator it;
 
@@ -132,7 +132,7 @@ namespace nux
     if (n_elements != 0)
     {
       // Compute the space available for each element
-      per_element_space = (remaining_height - total_used_space) / t_s32 (n_elements);
+      per_element_space = (remaining_height - total_used_space) / int (n_elements);
     }
 
     if (per_element_space < 0)
@@ -140,7 +140,7 @@ namespace nux
       per_element_space = 0;
     }
 
-    t_s32 margin;
+    int margin;
 
     if (per_element_space > 0)
     {
@@ -210,7 +210,7 @@ namespace nux
       (*it)->SetLayoutDone (false);
     }
 
-    t_s32 original_width = GetBaseWidth();
+    int original_width = GetBaseWidth();
 
     if (GetStretchFactor() == 0)
     {
@@ -253,24 +253,24 @@ namespace nux
     do
     {
       // Get layout Width and Height
-      t_s32 width = GetBaseWidth();
-      t_s32 height = GetBaseHeight();
+      int width = GetBaseWidth();
+      int height = GetBaseHeight();
 
       // Remove the margins. This is the real width and height available to the children.
       width -= 2 * m_h_out_margin;
-      height -= (t_s32) (num_element - 1) * m_v_in_margin + 2 * m_v_out_margin;
+      height -= (int) (num_element - 1) * m_v_in_margin + 2 * m_v_out_margin;
 
       // Size the children according to their stretch factor.
       VLayoutManagement (width, height);
 
       // Objects have been resized, now position them.
-      t_s32 current_x = GetBaseX() + m_h_out_margin;
-      t_s32 current_y = GetBaseY() + m_v_out_margin;
+      int current_x = GetBaseX() + m_h_out_margin;
+      int current_y = GetBaseY() + m_v_out_margin;
 
-      //t_s32 per_element_space = 0;
-      //t_s32 total_used_space = 0;
-      t_s32 offset_space = 0;
-      t_s32 space_after_element = 0;
+      //int per_element_space = 0;
+      //int total_used_space = 0;
+      int offset_space = 0;
+      int space_after_element = 0;
       ComputeStacking (height, offset_space, space_after_element);
       current_y += offset_space;
 
@@ -295,7 +295,7 @@ namespace nux
           {
             // The size of the element in the minor dimension is a percentage of layout minor dimension size.
             // Note that children of the processed element may force it to have a bigger size.
-            t_s32 percentage_width = (width * percentage) / 100.0f;
+            int percentage_width = (width * percentage) / 100.0f;
             (*it)->SetBaseWidth (percentage_width);
             break;
           }
@@ -325,7 +325,7 @@ namespace nux
         // Compute the position of an element in the minor dimension.
         if ( (extend != eFull) || ( (*it)->GetBaseWidth() < width) )
         {
-          t_s32 widget_width = (*it)->GetBaseWidth();
+          int widget_width = (*it)->GetBaseWidth();
 
           switch (positioning)
           {
@@ -363,11 +363,11 @@ namespace nux
       m_contentWidth = GetBaseWidth() - 2 * m_h_out_margin; // Set to the size of the layout.
 
       if (num_element == 0)
-        m_fittingHeight = (t_s32) 2 * m_v_out_margin;
+        m_fittingHeight = (int) 2 * m_v_out_margin;
       else
-        m_fittingHeight = (t_s32) (num_element - 1) * m_v_in_margin + 2 * m_v_out_margin;
+        m_fittingHeight = (int) (num_element - 1) * m_v_in_margin + 2 * m_v_out_margin;
 
-      t_s32 element_width;
+      int element_width;
       unadjusted_layout = false;
 
       // This array is meant to store the sizes of some of the elements width. These elements must have eFull as extent and
@@ -375,15 +375,15 @@ namespace nux
       // and therefore its size has been set to 1x1 at the start of this function, there is a possibility that some of the elements don't have
       // the full width of the layout(these elements uses their minimum width because the layout was set to a size 1x1).
       // We check if that is the case and force a recompute.
-      std::vector<t_s32> FullSizeUnadjusted;
+      std::vector<int> FullSizeUnadjusted;
 
       for (it = _layout_element_list.begin(); it != _layout_element_list.end(); it++)
       {
-        bool largerHeight = false;
-        bool smallerHeight = false;
-        bool largerWidth = false;
-        bool smallerWidth = false;
-        t_s32 ret = 0;
+        bool larger_height  = false;
+        bool smaller_height = false;
+        bool smaller_width  = false;
+        bool larger_width   = false;
+        int ret = 0;
 
         if (!(*it)->IsVisible ())
           continue;
@@ -392,12 +392,12 @@ namespace nux
         {
           ret = (*it)->ComputeLayout2();
 
-          largerHeight = (ret & eLargerHeight) ? true : false;
-          smallerHeight = (ret & eSmallerHeight) ? true : false;
-          largerWidth = (ret & eLargerWidth) ? true : false;
-          smallerWidth = (ret & eSmallerWidth) ? true : false;
+          larger_height   = (ret & eLargerHeight)   ? true : false;
+          smaller_height  = (ret & eSmallerHeight)  ? true : false;
+          smaller_width   = (ret & eSmallerWidth)   ? true : false;
+          larger_width    = (ret & eLargerWidth)    ? true : false;
 
-          if ( (largerHeight || smallerHeight) && ( (*it)->IsLayoutDone() == false) )
+          if ( (larger_height || smaller_height) && ( (*it)->IsLayoutDone() == false) )
           {
             // Stop computing the size of this layout. Its size was not convenient to its children. So the children size take priority
             // over the layout. In ComputeLayout2, the dimension of the layout has been set so it encompasses its children (and the margins).
@@ -405,7 +405,7 @@ namespace nux
             // of this layout is now 0.
             // This is the only place where a layout can have _layout_done set to "true".
 
-            // If (smallerHeight == true) the layout takes less space than anticipated.
+            // If (smaller_height == true) the layout takes less space than anticipated.
             // Set unadjusted_layout = true, so another pass will allow its sibling to claim more space.
             {
               unadjusted_layout = true;
@@ -413,9 +413,9 @@ namespace nux
             }
           }
 
-          if ( (smallerWidth == false) && ( (*it)->GetExtend() == eFull) && ( (*it)->GetBaseWidth() < (*it)->GetMaximumWidth() ) )
+          if ( (smaller_width == false) && ( (*it)->GetExtend() == eFull) && ( (*it)->GetBaseWidth() < (*it)->GetMaximumWidth() ) )
           {
-            // We catch all object whose size is possibly larger than the layout. We check there size at the end and
+            // Catch all object whose size is possibly larger than the layout. Check their size at the end and
             // recompute the layout if necessary.
             // For layout elements, make sure that the stretch factor is not 0. If it is, it means it will not use the
             // size provided by the parent layout. Its size will be adjusted to the minimum size of the layout content.
@@ -423,6 +423,12 @@ namespace nux
               FullSizeUnadjusted.push_back ( (*it)->GetBaseWidth() );
           }
 
+          if ((smaller_width || larger_width) && ((*it)->GetExtend() == MINOR_SIZE_MATCHCONTENT))
+          {
+            (*it)->SetMinimumWidth((*it)->GetBaseWidth());
+            unadjusted_layout = true;
+          }
+
           // Should be reactivate so that if the parent Layout does not call
           // ComputePosition2, at least it is done here to arrange the internal
           // element of the children.
@@ -467,8 +473,8 @@ namespace nux
       // m_contentWidth + 2 * m_h_out_margin;
       SetBaseWidth (m_contentWidth + 2 * m_h_out_margin);
 
-      t_s32 temp = m_contentWidth;
-      std::vector<t_s32>::iterator IntIterator = FullSizeUnadjusted.begin();
+      int temp = m_contentWidth;
+      std::vector<int>::iterator IntIterator = FullSizeUnadjusted.begin();
 
       for (IntIterator = FullSizeUnadjusted.begin(); IntIterator != FullSizeUnadjusted.end(); IntIterator++)
       {
@@ -534,14 +540,14 @@ namespace nux
     return size_compliance;
   }
 
-  void VLayout::VLayoutManagement (t_s32 width, t_s32 height)
+  void VLayout::VLayoutManagement (int width, int height)
   {
     bool need_recompute = false;
 
     do
     {
-      t_s32 available_height = height;
-      t_u32 max_stretchfactor = getMaxStretchFactor();
+      int available_height = height;
+      int max_stretchfactor = GetMaxStretchFactor();
       std::list<Area *>::iterator it;
 
       for (it = _layout_element_list.begin(); it != _layout_element_list.end(); it++)
@@ -606,7 +612,7 @@ namespace nux
 
       float cumul = 0;
       Area *LastElementThatCanBeResized = 0;
-      t_s32 total_distributed_size = 0;
+      int total_distributed_size = 0;
 
       for (it = _layout_element_list.begin(); it != _layout_element_list.end(); it++)
       {
@@ -633,7 +639,7 @@ namespace nux
       //                  --
       //                  all element with stretchfactor != 0
 
-      t_u32 ref_height = available_height / cumul;
+      int ref_height = available_height / cumul;
 
       //
       need_recompute = false;;
@@ -645,8 +651,8 @@ namespace nux
 
         if ( ( (*it)->GetStretchFactor() != 0) && ( (*it)->IsLayoutDone() == false) )
         {
-          t_u32 sf = (*it)->GetStretchFactor();
-          t_u32 new_height;
+          int sf = (*it)->GetStretchFactor();
+          int new_height;
 
           if (sf == max_stretchfactor)
           {
@@ -672,8 +678,8 @@ namespace nux
             }
           }
 
-          t_u32 elemt_max_height = (*it)->GetMaximumSize().GetHeight();
-          t_u32 elemt_min_height = (*it)->GetMinimumSize().GetHeight();
+          int elemt_max_height = (*it)->GetMaximumSize().height;
+          int elemt_min_height = (*it)->GetMinimumSize().height;
 
           // A layout must never have _layout_done set to true "here" because it must continue
           // doing the layout of its children and finally resize itself to fit them.
@@ -728,7 +734,7 @@ namespace nux
     while (need_recompute);
   }
 
-  t_u32 VLayout::getMaxStretchFactor()
+  t_u32 VLayout::GetMaxStretchFactor()
   {
     t_u32 value = 0;
     t_u32 sf;
@@ -767,18 +773,18 @@ namespace nux
           num_element++;
       }
       // Get layout Width and Height
-      t_s32 width = GetBaseWidth();
-      t_s32 height = GetBaseHeight();
+      int width = GetBaseWidth();
+      int height = GetBaseHeight();
       // remove the margins
       width -= 2 * m_h_out_margin;
-      height -= (t_s32) (num_element - 1) * m_v_in_margin + 2 * m_v_out_margin;
+      height -= (int) (num_element - 1) * m_v_in_margin + 2 * m_v_out_margin;
 
       // Objects have been resized, now position them.
-      t_s32 current_x = GetBaseX() + m_h_out_margin + offsetX; // add base offset in X (used for scrolling)
-      t_s32 current_y = GetBaseY() + m_v_out_margin + offsetY; // add base offset in Y (used for scrolling)
+      int current_x = GetBaseX() + m_h_out_margin + offsetX; // add base offset in X (used for scrolling)
+      int current_y = GetBaseY() + m_v_out_margin + offsetY; // add base offset in Y (used for scrolling)
 
-      t_s32 offset_space = 0;
-      t_s32 element_margin = 0;
+      int offset_space = 0;
+      int element_margin = 0;
       ComputeStacking (height, offset_space, element_margin);
       current_y += offset_space;
 
@@ -796,7 +802,7 @@ namespace nux
 
         if ( (extend != eFull) || ( (*it)->GetBaseWidth() < width) )
         {
-          t_s32 widget_width = (*it)->GetBaseWidth();
+          int widget_width = (*it)->GetBaseWidth();
 
           switch (positioning)
           {
@@ -844,7 +850,120 @@ namespace nux
       {
         (*it)->PositionChildLayout (offsetX, offsetY);
       }
-
     }
   }
+
+  Area* VLayout::KeyNavIteration(KeyNavDirection direction)
+  {
+    if (_layout_element_list.size() == 0)
+      return NULL;
+
+    if (IsVisible() == false)
+      return NULL;
+
+    if (next_object_to_key_focus_area_)
+    {
+      if ((direction == KEY_NAV_LEFT) || (direction == KEY_NAV_RIGHT))
+      {
+        // Don't know what to do with this
+        return NULL;
+      }
+      std::list<Area*>::iterator it;
+      std::list<Area*>::iterator it_next;
+      it = std::find (_layout_element_list.begin(), _layout_element_list.end(), next_object_to_key_focus_area_);
+
+      if (it == _layout_element_list.end())
+      {
+        // Should never happen
+        nuxAssert (0);
+        return NULL;
+      }
+
+      it_next = it;
+      ++it_next;
+
+      if ((direction == KEY_NAV_UP) && (it == _layout_element_list.begin()))
+      {
+        // can't go further
+        return NULL;
+      }
+
+      if ((direction == KEY_NAV_DOWN) && (it_next == _layout_element_list.end()))
+      {
+        // can't go further
+        return NULL;
+      }
+
+      if (direction == KEY_NAV_UP)
+      {
+        --it;
+        Area* key_nav_focus = (*it)->KeyNavIteration(direction);
+
+        while (key_nav_focus == NULL)
+        {
+          if (it == _layout_element_list.begin())
+            break;
+
+          --it;
+          key_nav_focus = (*it)->KeyNavIteration(direction);
+        }
+
+        return key_nav_focus;
+      }
+
+      if (direction == KEY_NAV_DOWN)
+      {
+        ++it;
+        Area* key_nav_focus = (*it)->KeyNavIteration(direction);
+
+        while (key_nav_focus == NULL)
+        {
+          ++it;
+          if (it == _layout_element_list.end())
+            break;
+
+          key_nav_focus = (*it)->KeyNavIteration(direction);
+        }
+
+        return key_nav_focus;
+      }
+    }
+    else
+    {
+      Area* key_nav_focus = NULL;
+
+      if (direction == KEY_NAV_UP)
+      {
+        std::list<Area*>::reverse_iterator it = _layout_element_list.rbegin();
+        key_nav_focus = (*it)->KeyNavIteration(direction);
+
+        while (key_nav_focus == NULL)
+        {
+          ++it;
+          if (it == _layout_element_list.rend())
+            break;
+
+          key_nav_focus = (*it)->KeyNavIteration(direction);
+        }
+      }
+      else
+      {
+        std::list<Area*>::iterator it = _layout_element_list.begin();
+        key_nav_focus = (*it)->KeyNavIteration(direction);
+
+        while (key_nav_focus == NULL)
+        {
+          ++it;
+          if (it == _layout_element_list.end())
+            break;
+
+          key_nav_focus = (*it)->KeyNavIteration(direction);
+        }
+      }
+
+      return key_nav_focus;
+    }
+
+    return NULL;
+  }
 }
