@@ -22,6 +22,7 @@
 
 #include "Nux.h"
 #include "Layout.h"
+#include "NuxCore/Logger.h"
 #include "NuxGraphics/GraphicsEngine.h"
 #include "ClientArea.h"
 #include "WindowCompositor.h"
@@ -33,7 +34,11 @@
 
 namespace nux
 {
-  
+namespace
+{
+logging::Logger logger("nux.windows.thread");
+}
+
     TimerFunctor *m_ScrollTimerFunctor;
     TimerHandle m_ScrollTimerHandler;  
 
@@ -76,7 +81,7 @@ namespace nux
     MSG msg;
     retval = PeekMessageW (&msg, NULL, 0, 0, PM_NOREMOVE) ? TRUE : FALSE;
 #elif defined(NUX_OS_LINUX)
-    retval = GetThreadGLWindow()->HasXPendingEvent () ? TRUE : FALSE;
+    retval = GetGraphicsDisplay()->HasXPendingEvent () ? TRUE : FALSE;
 #else
 #error Not implemented.
 #endif
@@ -99,7 +104,7 @@ namespace nux
       MSG msg;
       retval = PeekMessageW (&msg, NULL, 0, 0, PM_NOREMOVE) ? TRUE : FALSE;
 #elif defined(NUX_OS_LINUX)
-      retval = GetThreadGLWindow()->HasXPendingEvent () ? TRUE : FALSE;
+      retval = GetGraphicsDisplay()->HasXPendingEvent () ? TRUE : FALSE;
 #else
 #error Not implemented.
 #endif
@@ -235,7 +240,7 @@ namespace nux
 #if defined(NUX_OS_WINDOWS)
     event_source->event_poll_fd.fd = G_WIN32_MSG_HANDLE;
 #elif defined(NUX_OS_LINUX)
-    event_source->event_poll_fd.fd = ConnectionNumber (GetThreadGLWindow()->GetX11Display() );
+    event_source->event_poll_fd.fd = ConnectionNumber (GetGraphicsDisplay()->GetX11Display() );
 #else
 #error Not implemented.
 #endif
@@ -351,8 +356,9 @@ namespace nux
     {
       if((m_GLibContext == 0) || (m_GLibLoop == 0))
       {
-          nuxDebugMsg(TEXT("[WindowThread::AddGLibTimeout] WARNING: Trying to set a timeout before GLib Context is created."));
-          return 0;
+        LOG_WARNING(logger) << "Trying to set a timeout before GLib Context is created.\n"
+                            << logging::backtrace();
+        return 0;
       }
 
       GSource *timeout_source;
@@ -425,6 +431,10 @@ namespace nux
     m_GLibLoop      = 0;
     m_GLibContext   = 0;
 #endif
+#if defined(NUX_OS_LINUX)
+    _x11display = NULL;
+    _ownx11display = false;
+#endif
     
     _pending_wake_up_timer = false;
     _inside_main_loop = false;
@@ -455,6 +465,13 @@ namespace nux
     delete _Timelines;
     delete _async_wake_up_functor;
     delete _fake_event_call_back;
+
+#if defined(NUX_OS_LINUX)
+    if (_x11display && _ownx11display)
+    {
+      XCloseDisplay(_x11display);
+    }
+#endif
   }
 
   void WindowThread::SetFocusedArea (Area *focused_area)
@@ -474,7 +491,7 @@ namespace nux
     }
     
     _focused_area = focused_area;
-    _focused_area_destroyed_con = focused_area->OnDestroyed.connect (sigc::mem_fun (this, &WindowThread::OnFocusedAreaDestroyed));
+    _focused_area_destroyed_con = focused_area->object_destroyed.connect (sigc::mem_fun (this, &WindowThread::OnFocusedAreaDestroyed));
     
   }
 
@@ -892,8 +909,6 @@ namespace nux
 #endif
   {
     IEvent event;
-    float ms;
-    bool KeepRunning = true;
 
     if (!IsEmbeddedWindow() && GetWindow().IsPauseThreadGraphicsRendering() )
     {
@@ -904,23 +919,17 @@ namespace nux
     WindowThread *Application = GetWindowThread ();
 
 #if (!defined(NUX_OS_LINUX) && !defined(NUX_USE_GLIB_LOOP_ON_WINDOWS)) || defined(NUX_DISABLE_GLIB_LOOP)
-    while (KeepRunning)
+    while (true)
 #endif
     {
       _inside_main_loop = true;
       if(Application->m_bFirstDrawPass)
       {
-        ms = 0.0f;
         GetTimer().StartEarlyTimerObjects ();
       }
-      else
-      {
-        ms = GetWindow().GetFrameTime();
-      }
 
       memset(&event, 0, sizeof(IEvent));
       GetWindow().GetSystemEvent(&event);
-      
 
       if ((event.e_event == NUX_DND_ENTER_WINDOW) ||
         (event.e_event == NUX_DND_LEAVE_WINDOW))
@@ -961,8 +970,7 @@ namespace nux
 
       if((event.e_event ==	NUX_TERMINATE_APP) || (this->GetThreadState() == THREADSTOP))
       {
-          KeepRunning = false;
-          return 0; //break;
+          return 0;
       }
       
       if (IsEmbeddedWindow () && (event.e_event == NUX_SIZE_CONFIGURATION))
@@ -988,7 +996,7 @@ namespace nux
           (event.e_event == NUX_DND_DROP) ||
           (event.e_event == NUX_DND_ENTER) ||
           (event.e_event == NUX_DND_LEAVE) ||
-          (event.e_event == NUX_MOUSEWHEEL))
+          (event.e_event == NUX_MOUSE_WHEEL))
       {
           //DISPATCH EVENT HERE
           //event.Application = Application;
@@ -1449,7 +1457,6 @@ namespace nux
     m_Painter = new BasePainter();
     m_TimerHandler = new TimerHandler();
     m_window_compositor = new WindowCompositor;
-    m_Theme = new UXTheme();
 
     SetThreadState (THREADRUNNING);
     m_ThreadCtorCalled = true;
@@ -1502,7 +1509,6 @@ namespace nux
     m_Painter = new BasePainter();
     m_TimerHandler = new TimerHandler();
     m_window_compositor = new WindowCompositor;
-    m_Theme = new UXTheme();
 
     SetThreadState (THREADRUNNING);
     m_ThreadCtorCalled = true;
@@ -1530,18 +1536,19 @@ namespace nux
     }
 
     inlSetThreadLocalStorage (ThreadLocal_InalogicAppImpl, this);
-    GraphicsDisplay *ParentWindow = 0;
 
-    if (m_Parent && static_cast<WindowThread *> (m_Parent)->Type().IsObjectType (WindowThread::StaticObjectType) )
+    if (X11Display)
     {
-      ParentWindow = &static_cast<WindowThread *> (m_Parent)->GetWindow();
+      _x11display = X11Display;
+      _ownx11display = false;
     }
     else
     {
-      ParentWindow = 0;
+      _x11display = XOpenDisplay(NULL);
+      _ownx11display = true;
     }
-
-    _graphics_display = gGLWindowManager.CreateFromForeignWindow (X11Display, X11Window, OpenGLContext);
+      
+    _graphics_display = gGLWindowManager.CreateFromForeignWindow (_x11display, X11Window, OpenGLContext);
 
     if (_graphics_display == 0)
     {
@@ -1558,7 +1565,6 @@ namespace nux
     m_Painter = new BasePainter();
     m_TimerHandler = new TimerHandler();
     m_window_compositor = new WindowCompositor;
-    m_Theme = new UXTheme();
 
     SetThreadState (THREADRUNNING);
     m_ThreadCtorCalled = true;
@@ -1738,7 +1744,7 @@ namespace nux
         (nux_event.e_event == NUX_WINDOW_ENTER_FOCUS) ||
         (nux_event.e_event == NUX_WINDOW_EXIT_FOCUS) ||
         (nux_event.e_event == NUX_WINDOW_MOUSELEAVE) ||
-        (nux_event.e_event == NUX_MOUSEWHEEL))
+        (nux_event.e_event == NUX_MOUSE_WHEEL))
     {
         //DISPATCH EVENT HERE
         //nux_event.Application = Application;
@@ -1832,6 +1838,8 @@ namespace nux
 
     if (!IsEmbeddedWindow() )
       return;
+    
+    IOpenGLShaderProgram::SetShaderTracking(true);
 
     // Set Nux opengl states. The other plugin in compiz have changed the GPU opengl states.
     // Nux keep tracks of its own opengl states and restore them before doing any drawing.
@@ -1863,7 +1871,8 @@ namespace nux
 
     CHECKGL ( glColorMask (GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE) );
 
-    GetGpuDevice()->DeactivateFrameBuffer();
+    GetGraphicsDisplay()->GetGpuDevice()->DeactivateFrameBuffer();
+    IOpenGLShaderProgram::SetShaderTracking(false);
   }
 
   int WindowThread::InstallEventInspector (EventInspector function, void* data)
@@ -1954,5 +1963,17 @@ namespace nux
 
     return discard_event;
   }
+
+
+UXTheme& WindowThread::GetTheme() const
+{
+  if (!m_Theme)
+  {
+    LOG_INFO(logger) << "Lazily creating nux::UXTheme";
+    const_cast<WindowThread*>(this)->m_Theme = new UXTheme();
+  }
+  return *m_Theme;
+}
+
 }
 
