@@ -222,12 +222,12 @@ namespace nux
 #endif
 
           TextureRectangle *Texture = new TextureRectangle (NUX_TRACKER_LOCATION);
-          NBitmapData *BitmapData = LoadImageFile (font_texture_file.GetTCharPtr ());
+          NBitmapData* bitmap_data = LoadImageFile(font_texture_file.GetTCharPtr ());
 
-          if (BitmapData)
-            Texture->Update (BitmapData, false);
+          if (bitmap_data)
+            Texture->Update (bitmap_data, false);
 
-          NUX_SAFE_DELETE (BitmapData);
+          delete bitmap_data;
           TextureArray.push_back (Texture);
         }
       }
