@@ -53,7 +53,7 @@ namespace nux
   static T *UpCastResource (U *Src)
   {
     if (!Src || !Src->Type().IsDerivedFromType (T::StaticObjectType))
-      nuxError (TEXT ("[UpCastResource] Cast of %s to %s failed"), U::StaticObjectType.m_Name, T::StaticObjectType.m_Name);
+      nuxError (TEXT ("[UpCastResource] Cast of %s to %s failed"), U::StaticObjectType.name, T::StaticObjectType.name);
 
     return (T *) Src;
   }
@@ -137,7 +137,7 @@ namespace nux
     }
 
     // Create a 2D texture and upload the pixels.
-    BaseTexture *texture = GetGpuDevice ()->CreateSystemCapableTexture ();
+    BaseTexture *texture = GetGraphicsDisplay()->GetGpuDevice()->CreateSystemCapableTexture ();
     texture->Update (data);
     
     delete data; // data is deleted as texture->Update () copies it.
@@ -170,7 +170,7 @@ namespace nux
 
     if (BitmapData->IsTextureData() )
     {
-      BaseTexture *texture = GetGpuDevice()->CreateSystemCapableTexture();
+      BaseTexture *texture = GetGraphicsDisplay()->GetGpuDevice()->CreateSystemCapableTexture();
       texture->Update (BitmapData);
       return texture;
     }
@@ -180,37 +180,39 @@ namespace nux
 
   BaseTexture *CreateTextureFromFile (const TCHAR *TextureFilename)
   {
-    NBitmapData *BitmapData = LoadImageFile (TextureFilename);
+    BaseTexture* texture = nullptr;
+
+    NBitmapData* BitmapData = LoadImageFile (TextureFilename);
     NUX_RETURN_VALUE_IF_NULL (BitmapData, 0);
 
     if (BitmapData->IsTextureData() )
     {
-      BaseTexture *texture = GetGpuDevice()->CreateSystemCapableTexture();
-      texture->Update (BitmapData);
-      return texture;
+      texture = GetGraphicsDisplay()->GetGpuDevice()->CreateSystemCapableTexture();
     }
     else if (BitmapData->IsCubemapTextureData() )
     {
-      TextureCube *texture = new TextureCube();
-      texture->Update (BitmapData);
-      return texture;
+      texture = new TextureCube();
     }
     else if (BitmapData->IsVolumeTextureData() )
     {
-      TextureVolume *texture = new TextureVolume();
-      texture->Update (BitmapData);
-      return texture;
+      texture = new TextureVolume();
     }
     else if (BitmapData->IsAnimatedTextureData() )
     {
-      TextureFrameAnimation *texture = new TextureFrameAnimation();
-      texture->Update (BitmapData);
-      return texture;
+      texture = new TextureFrameAnimation();
+    }
+
+    if (texture)
+    {
+      texture->Update(BitmapData);
+    }
+    else
+    {
+      nuxDebugMsg ("[CreateTextureFromFile] Invalid texture format type for file (%s)", TextureFilename);
     }
 
     delete BitmapData;
-    nuxDebugMsg ("[CreateTextureFromFile] Invalid texture format type for file (%s)", TextureFilename);
-    return 0;
+    return texture;
   }
 
   BaseTexture *CreateTextureFromBitmapData (const NBitmapData *BitmapData)
@@ -220,7 +222,7 @@ namespace nux
 
     if (BitmapData->IsTextureData() )
     {
-      BaseTexture *texture = GetGpuDevice()->CreateSystemCapableTexture();
+      BaseTexture *texture = GetGraphicsDisplay()->GetGpuDevice()->CreateSystemCapableTexture();
       texture->Update (BitmapData);
       return texture;
     }
@@ -258,27 +260,30 @@ namespace nux
 
   ObjectPtr < IOpenGLBaseTexture > BaseTexture::GetDeviceTexture()
   {
-    ObjectPtr<CachedBaseTexture> CachedTexture = GetThreadGraphicsContext()->CacheResource (this);
+    ObjectPtr<CachedBaseTexture> CachedTexture = GetGraphicsDisplay()->GetGraphicsEngine()->CacheResource (this);
     return CachedTexture->m_Texture;
   }
 
   ObjectPtr<CachedBaseTexture> BaseTexture::GetCachedTexture()
   {
-    return GetThreadGraphicsContext()->CacheResource (this);
+    return GetGraphicsDisplay()->GetGraphicsEngine()->CacheResource(this);
   }
 
   Texture2D::Texture2D(NUX_FILE_LINE_DECL)
+    : BaseTexture(NUX_FILE_LINE_PARAM)
   {
   }
 
   Texture2D::Texture2D (const Texture2D &texture, NUX_FILE_LINE_DECL)
+    : BaseTexture(NUX_FILE_LINE_PARAM)
   {
     _image = texture._image;
   }
 
-  Texture2D::Texture2D (const NTextureData &BaseTexture, NUX_FILE_LINE_DECL)
+  Texture2D::Texture2D (const NTextureData &texture_data, NUX_FILE_LINE_DECL)
+    : BaseTexture(NUX_FILE_LINE_PARAM)
   {
-    _image = BaseTexture;
+    _image = texture_data;
   }
 
   Texture2D &Texture2D::operator = (const Texture2D &texture)
@@ -310,7 +315,7 @@ namespace nux
     if (UpdateAndCacheResource)
     {
       // call the texture manager and recreate the texture (CachedTexture2D) associated with this object if any.
-      GetThreadGraphicsContext()->UpdateResource (this);
+      GetGraphicsDisplay()->GetGraphicsEngine()->UpdateResource (this);
     }
 
     return true;
@@ -395,7 +400,7 @@ namespace nux
       return;
     }
 
-    m_Texture = GetGpuDevice()->CreateTexture (SourceTexture->GetWidth(),
+    m_Texture = GetGraphicsDisplay()->GetGpuDevice()->CreateTexture (SourceTexture->GetWidth(),
                 SourceTexture->GetHeight(),
                 SourceTexture->GetNumMipLevel(),
                 SourceTexture->GetFormat());
@@ -427,7 +432,7 @@ namespace nux
 
     if ( RecreateTexture (SourceTexture) )
     {
-      m_Texture = GetGpuDevice()->CreateTexture (SourceTexture->GetWidth(),
+      m_Texture = GetGraphicsDisplay()->GetGpuDevice()->CreateTexture (SourceTexture->GetWidth(),
                   SourceTexture->GetHeight(),
                   SourceTexture->GetNumMipLevel(),
                   SourceTexture->GetFormat() );
@@ -456,8 +461,6 @@ namespace nux
     OGL_CALL ( Texture2D->UnlockRect ( MipLevel ) );
   }
 
-//////////////////////////////////////////////////////////////////////
-
   TextureRectangle::TextureRectangle (NUX_FILE_LINE_DECL)
     : BaseTexture (NUX_FILE_LINE_PARAM)
   {
@@ -503,7 +506,7 @@ namespace nux
     if (UpdateAndCacheResource)
     {
       // call the texture manager and recreate the texture (CachedTexture2D) associated with this object if any.
-      GetThreadGraphicsContext()->UpdateResource (this);
+      GetGraphicsDisplay()->GetGraphicsEngine()->UpdateResource (this);
     }
     return true;
   }
@@ -549,7 +552,7 @@ namespace nux
       return;
     }
 
-    m_Texture = GetGpuDevice()->CreateRectangleTexture (SourceTexture->GetWidth(),
+    m_Texture = GetGraphicsDisplay()->GetGpuDevice()->CreateRectangleTexture (SourceTexture->GetWidth(),
                 SourceTexture->GetHeight(),
                 SourceTexture->GetNumMipLevel(),
                 SourceTexture->GetFormat() );
@@ -581,7 +584,7 @@ namespace nux
 
     if ( RecreateTexture (SourceTexture) )
     {
-      m_Texture = GetGpuDevice()->CreateRectangleTexture (SourceTexture->GetWidth(),
+      m_Texture = GetGraphicsDisplay()->GetGpuDevice()->CreateRectangleTexture (SourceTexture->GetWidth(),
                   SourceTexture->GetHeight(),
                   SourceTexture->GetNumMipLevel(),
                   SourceTexture->GetFormat() );
@@ -610,8 +613,6 @@ namespace nux
     OGL_CALL (TextureRectangle->UnlockRect ( MipLevel ) );
   }
 
-//////////////////////////////////////////////////////////////////////////
-
   TextureCube::TextureCube(NUX_FILE_LINE_DECL)
     : BaseTexture (NUX_FILE_LINE_PARAM)
   {
@@ -652,7 +653,7 @@ namespace nux
     if (UpdateAndCacheResource)
     {
       // call the texture manager and recreate the texture (CachedTexture2D) associated with this object if any.
-      GetThreadGraphicsContext()->UpdateResource (this);
+      GetGraphicsDisplay()->GetGraphicsEngine()->UpdateResource (this);
     }
 
     return true;
@@ -698,7 +699,7 @@ namespace nux
       return;
     }
 
-    m_Texture = GetGpuDevice()->CreateCubeTexture (SourceTexture->GetWidth(),
+    m_Texture = GetGraphicsDisplay()->GetGpuDevice()->CreateCubeTexture (SourceTexture->GetWidth(),
                 SourceTexture->GetNumMipLevel(),
                 SourceTexture->GetFormat() );
 
@@ -729,7 +730,7 @@ namespace nux
 
     if ( RecreateTexture (SourceTexture) )
     {
-      m_Texture = GetGpuDevice()->CreateCubeTexture (SourceTexture->GetWidth(),
+      m_Texture = GetGraphicsDisplay()->GetGpuDevice()->CreateCubeTexture (SourceTexture->GetWidth(),
                   SourceTexture->GetNumMipLevel(),
                   SourceTexture->GetFormat() );
 
@@ -760,8 +761,6 @@ namespace nux
     }
   }
 
-//////////////////////////////////////////////////////////////////////////
-
   TextureVolume::TextureVolume(NUX_FILE_LINE_DECL)
     : BaseTexture (NUX_FILE_LINE_PARAM)
   {
@@ -802,7 +801,7 @@ namespace nux
     if (UpdateAndCacheResource)
     {
       // call the texture manager and recreate the texture (CachedTexture2D) associated with this object if any.
-      GetThreadGraphicsContext()->UpdateResource (this);
+      GetGraphicsDisplay()->GetGraphicsEngine()->UpdateResource (this);
     }
 
     return true;
@@ -869,7 +868,7 @@ namespace nux
       return;
     }
 
-    m_Texture = GetGpuDevice()->CreateVolumeTexture (SourceTexture->GetWidth(),
+    m_Texture = GetGraphicsDisplay()->GetGpuDevice()->CreateVolumeTexture (SourceTexture->GetWidth(),
                 SourceTexture->GetHeight(),
                 SourceTexture->GetDepth(),
                 SourceTexture->GetNumMipLevel(),
@@ -902,7 +901,7 @@ namespace nux
 
     if ( RecreateTexture (SourceTexture) )
     {
-      m_Texture = GetGpuDevice()->CreateVolumeTexture (SourceTexture->GetWidth(),
+      m_Texture = GetGraphicsDisplay()->GetGpuDevice()->CreateVolumeTexture (SourceTexture->GetWidth(),
                   SourceTexture->GetHeight(),
                   SourceTexture->GetDepth(),
                   SourceTexture->GetNumMipLevel(),
@@ -943,7 +942,6 @@ namespace nux
 //     }
   }
 
-//////////////////////////////////////////////////////////////////////////
   TextureFrameAnimation::TextureFrameAnimation(NUX_FILE_LINE_DECL)
     : BaseTexture (NUX_FILE_LINE_PARAM)
   {
@@ -984,7 +982,7 @@ namespace nux
     if (UpdateAndCacheResource)
     {
       // call the texture manager and recreate the texture (CachedTexture2D) associated with this object if any.
-      GetThreadGraphicsContext()->UpdateResource (this);
+      GetGraphicsDisplay()->GetGraphicsEngine()->UpdateResource (this);
     }
 
     return true;
@@ -1040,7 +1038,7 @@ namespace nux
       return;
     }
 
-    m_Texture = GetGpuDevice()->CreateAnimatedTexture (SourceTexture->GetWidth(),
+    m_Texture = GetGraphicsDisplay()->GetGpuDevice()->CreateAnimatedTexture (SourceTexture->GetWidth(),
                 SourceTexture->GetHeight(),
                 SourceTexture->GetDepth(),
                 SourceTexture->GetFormat() );
@@ -1072,7 +1070,7 @@ namespace nux
 
     if (RecreateTexture (SourceTexture) )
     {
-      m_Texture = GetGpuDevice()->CreateAnimatedTexture (SourceTexture->GetWidth(),
+      m_Texture = GetGraphicsDisplay()->GetGpuDevice()->CreateAnimatedTexture (SourceTexture->GetWidth(),
                   SourceTexture->GetHeight(),
                   SourceTexture->GetDepth(),
                   SourceTexture->GetFormat() );
