@@ -106,6 +106,13 @@ namespace nux
     bool Support_ARB_Pixel_Buffer_Object ()       const    {return _support_arb_pixel_buffer_object;}
     bool Support_EXT_Blend_Equation_Separate ()   const    {return _support_ext_blend_equation_separate;}
 
+#ifndef NUX_OPENGLES_20
+    bool Support_EXT_Texture_sRGB ()              const    {return _support_ext_texture_srgb;}
+    bool Support_EXT_Texture_Decode ()            const    {return _support_ext_texture_srgb_decode;}
+    bool Support_EXT_Framebuffer_sRGB ()          const    {return _support_ext_framebuffer_srgb;}
+    bool Support_ARB_Framebuffer_sRGB ()          const    {return _support_arb_framebuffer_srgb;}
+#endif
+
     int GetMaxFboAttachment () {return _opengl_max_fb_attachment;}
 
 
@@ -149,6 +156,13 @@ namespace nux
     bool _support_arb_pixel_buffer_object;
     bool _support_ext_blend_equation_separate;
 
+#ifndef NUX_OPENGLES_20
+    bool _support_ext_texture_srgb;
+    bool _support_ext_texture_srgb_decode;
+    bool _support_ext_framebuffer_srgb;
+    bool _support_arb_framebuffer_srgb;
+#endif
+
     friend class GpuDevice;
   };
 
@@ -253,10 +267,16 @@ namespace nux
 
   public:
     ObjectPtr<IOpenGLTexture2D> CreateTexture (
-      int Width
-      , int Height
-      , int Levels
-      , BitmapFormat PixelFormat);
+      int Width,
+      int Height,
+      int Levels,
+      BitmapFormat PixelFormat);
+
+    ObjectPtr<IOpenGLTexture2D> CreateTexture2DFromID(int id,
+      int Width,
+      int Height,
+      int Levels,
+      BitmapFormat PixelFormat);
 
     ObjectPtr<IOpenGLRectangleTexture> CreateRectangleTexture (
       int Width
@@ -321,10 +341,6 @@ namespace nux
       ObjectPtr<IOpenGLIndexBuffer> IndexBuffer,
       ObjectPtr<IOpenGLVertexDeclaration> VertexDeclaration,
       PRIMITIVE_TYPE PrimitiveType,
-      int BaseVertexIndex,
-      int MinIndex,
-      int NumVertices,
-      int StartIndex,
       int PrimitiveCount
     );
 
@@ -348,9 +364,7 @@ namespace nux
       unsigned int StreamNumber,
       ObjectPtr<IOpenGLVertexBuffer> pStreamData,
       unsigned int OffsetInBytes,
-      unsigned int Stride,
-      unsigned int NumComponent = 0,
-      unsigned int ComponentFormat = 0);
+      unsigned int Stride);
 
     //! Setup a NULL vertex buffer
     void InvalidateVertexBuffer();
@@ -503,6 +517,8 @@ namespace nux
 
   public:
     
+    ObjectPtr<IOpenGLTexture2D> backup_texture0_;
+
 #if defined (NUX_OS_WINDOWS)
     GpuDevice (unsigned int DeviceWidth, unsigned int DeviceHeight, BitmapFormat DeviceFormat,
       HDC device_context,
