@@ -33,8 +33,6 @@
 
 #include "FontTexture.h"
 #include "FontRenderer.h"
-#include "UIColorTheme.h"
-
 #include "GraphicsEngine.h"
 
 namespace nux
@@ -141,6 +139,17 @@ namespace nux
   }
 
 
+  FxStructure::FxStructure()
+  {
+    src_texture = nux::GetGraphicsDisplay()->GetGpuDevice()->CreateTexture(1, 1, 1, nux::BITFMT_R8G8B8A8);
+    dst_texture = nux::GetGraphicsDisplay()->GetGpuDevice()->CreateTexture(1, 1, 1, nux::BITFMT_R8G8B8A8);
+    temp_texture = nux::GetGraphicsDisplay()->GetGpuDevice()->CreateTexture(1, 1, 1, nux::BITFMT_R8G8B8A8);
+  }
+
+  FxStructure::~FxStructure()
+  {
+  }
+
   GraphicsEngine::GraphicsEngine (GraphicsDisplay &GlWindow, bool create_rendering_data)
     :   _graphics_display (GlWindow)
   {
@@ -177,8 +186,8 @@ namespace nux
 
     bool opengl_14_support = true;
 
-    if ((_graphics_display.GetGpuDevice ()->GetOpenGLMajorVersion () == 1) &&
-      (_graphics_display.GetGpuDevice ()->GetOpenGLMinorVersion () < 4))
+    if ((_graphics_display.GetGpuDevice()->GetOpenGLMajorVersion () == 1) &&
+      (_graphics_display.GetGpuDevice()->GetOpenGLMinorVersion () < 4))
     {
       // OpenGL version is less than OpenGL 1.4
       opengl_14_support = false;
@@ -188,8 +197,8 @@ namespace nux
     {
 #ifndef NUX_OPENGLES_20
       if (UsingGLSLCodePath () &&
-        GetGpuDevice ()->GetGpuInfo ().Support_ARB_Fragment_Shader () &&
-        GetGpuDevice ()->GetGpuInfo ().Support_ARB_Vertex_Shader () &&
+        _graphics_display.GetGpuDevice()->GetGpuInfo ().Support_ARB_Fragment_Shader () &&
+        _graphics_display.GetGpuDevice()->GetGpuInfo ().Support_ARB_Vertex_Shader () &&
         opengl_14_support)
       {
         InitSlColorShader ();
@@ -208,11 +217,11 @@ namespace nux
 
         InitSl2TextureDepRead ();
 
-        InitSLHorizontalHQGaussFilter ();
-        InitSLVerticalHQGaussFilter ();
+        InitSLHorizontalHQGaussFilter (1);
+        InitSLVerticalHQGaussFilter (1);
       }
-      else if (GetGpuDevice ()->GetGpuInfo ().Support_ARB_Fragment_Shader () &&
-        GetGpuDevice ()->GetGpuInfo ().Support_ARB_Vertex_Program () &&
+      else if (_graphics_display.GetGpuDevice()->GetGpuInfo ().Support_ARB_Fragment_Shader () &&
+        _graphics_display.GetGpuDevice()->GetGpuInfo ().Support_ARB_Vertex_Program () &&
         opengl_14_support)
       {
         InitAsmColorShader ();
@@ -248,28 +257,28 @@ namespace nux
 #endif
 
 #if defined (NUX_OS_WINDOWS)
-      if (_normal_font == 0)
+      if (_normal_font.IsNull())
       {
         FontTexture* fnt = new FontTexture (GNuxGraphicsResources.FindResourceLocation (TEXT ("Fonts/Tahoma_size_8.txt"), true).GetTCharPtr(), NUX_TRACKER_LOCATION);
         _normal_font = ObjectPtr<FontTexture> (fnt);
         fnt->UnReference ();
       }
 
-      if (_bold_font == 0)
+      if (_bold_font.IsNull())
       {
         FontTexture* fnt = new FontTexture (GNuxGraphicsResources.FindResourceLocation (TEXT ("Fonts/Tahoma_size_8_bold.txt"), true).GetTCharPtr(), NUX_TRACKER_LOCATION);
         _bold_font = ObjectPtr<FontTexture> (fnt);
         fnt->UnReference ();
       }
 #else
-      if (_normal_font == 0)
+      if (_normal_font.IsNull())
       {
         FontTexture* fnt = new FontTexture (GNuxGraphicsResources.FindResourceLocation (TEXT ("Fonts/Ubuntu_size_10.txt"), true).GetTCharPtr(), NUX_TRACKER_LOCATION);
         _normal_font = ObjectPtr<FontTexture> (fnt);
         fnt->UnReference ();
       }
 
-      if (_bold_font == 0)
+      if (_bold_font.IsNull())
       {
         FontTexture* fnt = new FontTexture (GNuxGraphicsResources.FindResourceLocation (TEXT ("Fonts/Ubuntu_size_10_bold.txt"), true).GetTCharPtr(), NUX_TRACKER_LOCATION);
         _bold_font = ObjectPtr<FontTexture> (fnt);
@@ -286,12 +295,12 @@ namespace nux
       }
 
       if (gpu_info.Support_EXT_Framebuffer_Object ())
-        _offscreen_fbo = GetGpuDevice()->CreateFrameBufferObject ();
+        _offscreen_fbo = _graphics_display.GetGpuDevice()->CreateFrameBufferObject ();
 
-      _offscreen_color_rt0  = GetGpuDevice()->CreateTexture(2, 2, 1, BITFMT_R8G8B8A8);
-      _offscreen_color_rt1  = GetGpuDevice()->CreateTexture(2, 2, 1, BITFMT_R8G8B8A8);
-      _offscreen_color_rt2  = GetGpuDevice()->CreateTexture(2, 2, 1, BITFMT_R8G8B8A8);
-      _offscreen_color_rt3  = GetGpuDevice()->CreateTexture(2, 2, 1, BITFMT_R8G8B8A8);
+      _offscreen_color_rt0  = _graphics_display.GetGpuDevice()->CreateTexture(2, 2, 1, BITFMT_R8G8B8A8);
+      _offscreen_color_rt1  = _graphics_display.GetGpuDevice()->CreateTexture(2, 2, 1, BITFMT_R8G8B8A8);
+      _offscreen_color_rt2  = _graphics_display.GetGpuDevice()->CreateTexture(2, 2, 1, BITFMT_R8G8B8A8);
+      _offscreen_color_rt3  = _graphics_display.GetGpuDevice()->CreateTexture(2, 2, 1, BITFMT_R8G8B8A8);
     }
   }
 
@@ -309,9 +318,14 @@ namespace nux
 
   void GraphicsEngine::EvaluateGpuCaps ()
   {
-    if (_graphics_display.GetGpuDevice ()->GetGpuInfo ().Support_ARB_Vertex_Shader() &&
-      _graphics_display.GetGpuDevice ()->GetGpuInfo ().Support_ARB_Fragment_Shader() &&
-      _graphics_display.GetGpuDevice ()->GetGPUBrand () ==  GPU_BRAND_NVIDIA)
+#ifdef NUX_OS_WINDOWS
+    if (_graphics_display.GetGpuDevice()->GetGpuInfo().Support_ARB_Vertex_Shader() &&
+      _graphics_display.GetGpuDevice()->GetGpuInfo().Support_ARB_Fragment_Shader())
+#else
+    if (_graphics_display.GetGpuDevice()->GetGpuInfo().Support_ARB_Vertex_Shader() &&
+      _graphics_display.GetGpuDevice()->GetGpuInfo().Support_ARB_Fragment_Shader() &&
+      (_graphics_display.GetGpuDevice()->GetOpenGLMajorVersion() >= 2))
+#endif
     {
       NString renderer_string = ANSI_TO_TCHAR (NUX_REINTERPRET_CAST (const char *, glGetString (GL_RENDERER)));
       CHECKGL_MSG (glGetString (GL_RENDERER));
@@ -509,18 +523,18 @@ namespace nux
 
   void GraphicsEngine::DisableAllTextureMode (int TextureUnit)
   {
-    if ( (TextureUnit < GL_TEXTURE0) || (TextureUnit > GL_TEXTURE31) )
+    if ((TextureUnit < GL_TEXTURE0) || (TextureUnit > GL_TEXTURE31))
       return;
 
-    GetGpuDevice()->InvalidateTextureUnit (TextureUnit);
+    _graphics_display.GetGpuDevice()->InvalidateTextureUnit(TextureUnit);
   }
 
 //////////////////////
 // DRAW CLIPPING    //
 //////////////////////
-  void GraphicsEngine::PushClippingRectangle (Rect rect)
+  void GraphicsEngine::PushClippingRectangle(Rect rect)
   {
-    if (_graphics_display.m_DeviceFactory->GetCurrentFrameBufferObject().IsValid() )
+    if(_graphics_display.m_DeviceFactory->GetCurrentFrameBufferObject().IsValid())
     {
       // There is an active framebuffer set. Push the clipping rectangles to that fbo clipping stack.
       _graphics_display.m_DeviceFactory->GetCurrentFrameBufferObject()->PushClippingRegion (rect);
@@ -918,28 +932,17 @@ namespace nux
 
   void GraphicsEngine::PushIdentityModelViewMatrix ()
   {
-    _model_view_stack.push_back (Matrix4::IDENTITY ());
-
-    _model_view_matrix = Matrix4::IDENTITY ();
-
-    std::list<Matrix4>::iterator it;
-    for (it = _model_view_stack.begin (); it != _model_view_stack.end (); it++)
-    {
-      _model_view_matrix = (*it) * _model_view_matrix;
-    }
+    PushModelViewMatrix (Matrix4::IDENTITY());
   }
 
   void GraphicsEngine::PushModelViewMatrix (const Matrix4 &matrix)
   {
-    _model_view_stack.push_back (matrix);
+    if (_model_view_stack.empty())
+      _model_view_matrix = matrix;
+    else
+      _model_view_matrix = matrix * _model_view_stack.back();
 
-    _model_view_matrix = Matrix4::IDENTITY ();
-    
-    std::list<Matrix4>::iterator it;
-    for (it = _model_view_stack.begin (); it != _model_view_stack.end (); it++)
-    {
-      _model_view_matrix = (*it) * _model_view_matrix;
-    }
+    _model_view_stack.push_back (_model_view_matrix);
   }
 
   void GraphicsEngine::Push2DTranslationModelViewMatrix (float tx, float ty, float tz)
@@ -952,20 +955,16 @@ namespace nux
 
   bool GraphicsEngine::PopModelViewMatrix ()
   {
-    if (_model_view_stack.size () == 0)
+    if (!_model_view_stack.empty())
+      _model_view_stack.pop_back();
+
+    if (_model_view_stack.empty())
     {
       _model_view_matrix = Matrix4::IDENTITY ();
       return false;
     }
 
-    _model_view_stack.pop_back();
-
-    _model_view_matrix = Matrix4::IDENTITY ();
-    std::list<Matrix4>::iterator it;
-    for (it = _model_view_stack.begin (); it != _model_view_stack.end (); it++)
-    {
-      _model_view_matrix = (*it) * _model_view_matrix;
-    }
+    _model_view_matrix = _model_view_stack.back();
 
     return true;
   }
@@ -983,13 +982,10 @@ namespace nux
 
   void GraphicsEngine::ApplyModelViewMatrix ()
   {
-    _model_view_matrix = Matrix4::IDENTITY ();
-
-    std::list<Matrix4>::iterator it;
-    for (it = _model_view_stack.begin (); it != _model_view_stack.end (); it++)
-    {
-      _model_view_matrix = (*it) * _model_view_matrix;
-    }
+    if (_model_view_stack.empty())
+      _model_view_matrix = Matrix4::IDENTITY ();
+    else
+      _model_view_matrix = _model_view_stack.back();
   }
 
   Rect GraphicsEngine::ModelViewXFormRect (const Rect& rect)
@@ -1263,7 +1259,7 @@ namespace nux
     //CHECKGL (glEnable(GL_DEPTH_TEST));
     //CHECKGL (glDepthFunc(GL_ALWAYS));
 
-    QRP_Color(x, y, width, height, Colors::Black);
+    QRP_Color(x, y, width, height, color::Black);
 
     //CHECKGL( glDepthFunc(GL_LESS) );
     //CHECKGL( glDisable(GL_DEPTH_TEST) );
@@ -1289,18 +1285,16 @@ namespace nux
 
   bool GraphicsEngine::FlushCachedResourceData (ResourceData *Resource)
   {
-    bool cached = IsResourceCached (Resource);
-    if (cached == false)
+    if (!IsResourceCached(Resource))
       return false;
 
-    ResourceCache.FlushResourceId (Resource->GetResourceIndex ());
+    ResourceCache.FlushResourceId (Resource->GetResourceIndex());
     return true;
   }
 
   void GraphicsEngine::UpdateResource (ResourceData *Resource)
   {
     ObjectPtr< CachedResourceData > GLResource = ResourceCache.FindCachedResourceById (Resource->GetResourceIndex() ); //(CachedResourceData*)(*(ResourceCache.ResourceMap.find(Resource->ResourceIndex))).second;
-    bool bUpdated = FALSE;
 
     if (GLResource.IsValid() )
     {
@@ -1313,7 +1307,7 @@ namespace nux
         // Check if the updater is valid for updating the resource.
         if ( ResourceUpdater->UpdatesThisResource (Resource) )
         {
-          bUpdated = ResourceUpdater->UpdateResource (GLResource, Resource);
+          ResourceUpdater->UpdateResource(GLResource, Resource);
           break;
         }
       }
@@ -1327,24 +1321,34 @@ namespace nux
 
   void GraphicsEngine::SetFrameBufferHelper (
     ObjectPtr<IOpenGLFrameBufferObject>& fbo,
-    ObjectPtr<IOpenGLTexture2D>& colorbuffer,
-    ObjectPtr<IOpenGLTexture2D>& depthbuffer,
+    ObjectPtr<IOpenGLBaseTexture>& colorbuffer,
+    ObjectPtr<IOpenGLBaseTexture>& depthbuffer,
     int width, int height)
   {
-    //if ((colorbuffer->GetWidth() != width) || (depthbuffer->GetHeight() != height))
+    if ((colorbuffer.IsValid() == false) || (colorbuffer->GetWidth() != width) || (colorbuffer->GetHeight() != height))
     {
-      colorbuffer = GetGpuDevice ()->CreateTexture (width, height, 1, BITFMT_R8G8B8A8);
-      depthbuffer = GetGpuDevice ()->CreateTexture (width, height, 1, BITFMT_D24S8);
+      colorbuffer = _graphics_display.GetGpuDevice()->CreateTexture(width, height, 1, BITFMT_R8G8B8A8);
+    }
+
+    if ((depthbuffer.IsValid()) && ((depthbuffer->GetWidth() != width) || (depthbuffer->GetHeight() != height)))
+    {
+      // Generate a new depth texture only if a valid one was passed to this function.
+      depthbuffer = _graphics_display.GetGpuDevice()->CreateTexture(width, height, 1, BITFMT_D24S8);
     }
 
     fbo->FormatFrameBufferObject(width, height, BITFMT_R8G8B8A8);
-    fbo->SetRenderTarget (0, colorbuffer->GetSurfaceLevel (0));
-    fbo->SetDepthSurface (depthbuffer->GetSurfaceLevel (0));
-    fbo->Activate ();
-    fbo->EmptyClippingRegion ();
-    SetContext (0, 0, width, height);
-    SetViewport (0, 0, width, height);
-    Push2DWindow (width, height);
+    fbo->SetRenderTarget(0, colorbuffer->GetSurfaceLevel(0));
+    
+    if (depthbuffer.IsValid())
+      fbo->SetDepthSurface(depthbuffer->GetSurfaceLevel(0));
+    else
+      fbo->SetDepthSurface(ObjectPtr<IOpenGLSurface>(NULL));
+
+    fbo->Activate();
+    fbo->EmptyClippingRegion();
+    SetContext(0, 0, width, height);
+    SetViewport(0, 0, width, height);
+    Push2DWindow(width, height);
   }
 
   void GraphicsEngine::GaussianWeights(float **weights, float sigma, unsigned int num_tap)
