@@ -42,6 +42,9 @@
   #include "GLWindowManager.h"
 #endif
 
+#define NUX_MAX_GAUSSIAN_SIGMA 11
+#define NUX_MIN_GAUSSIAN_SIGMA 1
+
 namespace nux
 {
 
@@ -88,6 +91,16 @@ namespace nux
 
   } OpenGLContext;
 
+  class FxStructure
+  {
+  public:
+    FxStructure();
+    ~FxStructure();
+    ObjectPtr<IOpenGLBaseTexture> dst_texture;
+    ObjectPtr<IOpenGLBaseTexture> src_texture;
+    ObjectPtr<IOpenGLBaseTexture> temp_texture;
+  };
+
   //! Rendering engine class
   class GraphicsEngine
   {
@@ -123,7 +136,6 @@ namespace nux
     void DisableTextureMode (int TextureUnit, int TextureMode);
     void DisableAllTextureMode (int TextureUnit);
 
-
     ///////////////////
     // DRAW TEXTURE  //
     ///////////////////
@@ -194,6 +206,12 @@ namespace nux
 	  ObjectPtr<IOpenGLBaseTexture> QRP_GetPixelBlocks (
 		  ObjectPtr<IOpenGLBaseTexture> device_texture, TexCoordXForm &texxform0, const Color& c0, int pixel_size);
 
+    void QRP_GetCopyTexture (
+      int width, int height,
+      ObjectPtr<IOpenGLBaseTexture>& dst_device_texture,
+      ObjectPtr<IOpenGLBaseTexture>& src_device_texture,
+      TexCoordXForm &texxform0, const Color& c0);
+
     // ASM
     void QRP_ASM_1Tex (int x, int y, int width, int height, ObjectPtr<IOpenGLBaseTexture> Tex0, TexCoordXForm &texxform, const Color &color0);
     void QRP_ASM_Pixelate (int x, int y, int width, int height, ObjectPtr<IOpenGLBaseTexture> DeviceTexture, TexCoordXForm &texxform, const Color &c0, int pixel_size);
@@ -243,6 +261,10 @@ namespace nux
       int x, int y, int buffer_width, int buffer_height,
       ObjectPtr<IOpenGLBaseTexture> device_texture, TexCoordXForm &texxform, const Color& color, float sigma = 1.0f, int num_pass = 1);
 
+    void QRP_ASM_GetBlurTextureFx (
+      int x, int y, int buffer_width, int buffer_height,
+      FxStructure *fx_structure, TexCoordXForm &texxform, const Color& color, float sigma = 1.0f, int num_pass = 1);
+
     //! Replicate the alpha channel in all components of the texture.
     /*!
         @param device_texture Source texture.
@@ -302,6 +324,12 @@ namespace nux
     ObjectPtr<IOpenGLBaseTexture>  QRP_ASM_GetPixelBlocks (
 		  ObjectPtr<IOpenGLBaseTexture> device_texture, TexCoordXForm &texxform0, const Color& color, int pixel_size);
 
+    void QRP_ASM_GetCopyTexture (
+      int width, int height,
+      ObjectPtr<IOpenGLBaseTexture>& dst_device_texture,
+      ObjectPtr<IOpenGLBaseTexture>& src_device_texture,
+      TexCoordXForm &texxform0, const Color& c0);
+
     // GLSL
 
     void QRP_GLSL_1Tex (int x, int y, int width, int height, ObjectPtr<IOpenGLBaseTexture> DeviceTexture, TexCoordXForm &texxform, const Color &c0);
@@ -380,11 +408,27 @@ namespace nux
       ObjectPtr<IOpenGLBaseTexture> device_texture, TexCoordXForm &texxform,
       const Color& c0, float sigma = 1.0f, int num_pass = 1);
 
+    void QRP_GLSL_GetHQBlurFx (
+      int x, int y,
+      int buffer_width, int buffer_height,
+      FxStructure *fx_structure, TexCoordXForm &texxform,
+      const Color& c0, float sigma = 1.0f, int num_pass = 1);
+
     void QRP_GLSL_DisturbedTexture (
       int x, int y, int width, int height,
       ObjectPtr<IOpenGLBaseTexture> distorsion_texture, TexCoordXForm &texxform0, const Color& c0,
       ObjectPtr<IOpenGLBaseTexture> src_device_texture, TexCoordXForm &texxform1, const Color& c1);
 
+    ObjectPtr<IOpenGLBaseTexture> QRP_GLSL_GetDisturbedTexture(
+      int x, int y, int width, int height,
+      ObjectPtr<IOpenGLBaseTexture> distorsion_texture, TexCoordXForm &texxform0, const Color& c0,
+      ObjectPtr<IOpenGLBaseTexture> src_device_texture, TexCoordXForm &texxform1, const Color& c1);
+
+    void QRP_GLSL_GetDisturbedTextureFx(
+      int x, int y, int width, int height,
+      ObjectPtr<IOpenGLBaseTexture> distorsion_texture, TexCoordXForm &texxform0, const Color& c0,
+      FxStructure *fx_structure, TexCoordXForm &texxform1, const Color& c1);
+
     //! Pixel blocks.
     /*!
         @param device_texture Source texture.
@@ -396,6 +440,12 @@ namespace nux
 	  ObjectPtr<IOpenGLBaseTexture> QRP_GLSL_GetPixelBlocks (
 		  ObjectPtr<IOpenGLBaseTexture> device_texture, TexCoordXForm &texxform0, const Color& c0, int pixel_size);
 
+    void QRP_GLSL_GetCopyTexture(
+      int width, int height,
+      ObjectPtr<IOpenGLBaseTexture>& dst_device_texture,
+      ObjectPtr<IOpenGLBaseTexture>& src_device_texture,
+      TexCoordXForm &texxform0, const Color& c0);
+
     //////////////////////
     // DRAW CLIPPING    //
     //////////////////////
@@ -669,8 +719,8 @@ namespace nux
     //! Helper function to set an fbo
     void SetFrameBufferHelper(
       ObjectPtr<IOpenGLFrameBufferObject>& fbo,
-      ObjectPtr<IOpenGLTexture2D>& colorbuffer,
-      ObjectPtr<IOpenGLTexture2D>& depthbuffer,
+      ObjectPtr<IOpenGLBaseTexture>& colorbuffer,
+      ObjectPtr<IOpenGLBaseTexture>& depthbuffer,
       int width, int height);
 
 #ifndef NUX_OPENGLES_20
@@ -815,13 +865,13 @@ namespace nux
     ObjectPtr<IOpenGLShaderProgram> _vertical_gauss_filter_prog;
     
 
-    void InitSLHorizontalHQGaussFilter ();
+    void InitSLHorizontalHQGaussFilter (int sigma);
     //! Gauss horizontal filter.
-    ObjectPtr<IOpenGLShaderProgram> _horizontal_hq_gauss_filter_prog;
+    ObjectPtr<IOpenGLShaderProgram> _horizontal_hq_gauss_filter_prog[NUX_MAX_GAUSSIAN_SIGMA];
 
-    void InitSLVerticalHQGaussFilter ();
+    void InitSLVerticalHQGaussFilter (int sigma);
     //! Gauss vertical filter.
-    ObjectPtr<IOpenGLShaderProgram> _vertical_hq_gauss_filter_prog;
+    ObjectPtr<IOpenGLShaderProgram> _vertical_hq_gauss_filter_prog[NUX_MAX_GAUSSIAN_SIGMA];
 
 
 
@@ -831,10 +881,14 @@ namespace nux
 
     void InitSlBlendModes();
 
-    ObjectPtr<IOpenGLShaderProgram> m_SlPSBNormal;
-    ObjectPtr<IOpenGLShaderProgram> m_SlPSBLighten;
-    ObjectPtr<IOpenGLShaderProgram> m_SlPSBDarken;
-    ObjectPtr<IOpenGLShaderProgram> m_SlPSBMultiply;
+    void InitBlendModeNormal();
+    void InitBlendModeLighten();
+    void InitBlendModeDarken();
+
+    ObjectPtr<IOpenGLShaderProgram> _shader_layer_blend_normal;
+    ObjectPtr<IOpenGLShaderProgram> _shader_layer_blend_lighten;
+    ObjectPtr<IOpenGLShaderProgram> _shader_layer_blend_darken;
+    ObjectPtr<IOpenGLShaderProgram> _shader_layer_blend_multiply;
 
 
     //! Test the gpu features and set variables such as \e _use_glsl_shaders.
@@ -843,14 +897,14 @@ namespace nux
     bool _use_glsl_shaders; //!< True if the system is using the glsl code path.
 
     ObjectPtr<IOpenGLFrameBufferObject> _offscreen_fbo;
-    ObjectPtr<IOpenGLTexture2D> _offscreen_color_rt0;
-    ObjectPtr<IOpenGLTexture2D> _offscreen_depth_rt0;
-    ObjectPtr<IOpenGLTexture2D> _offscreen_color_rt1;
-    ObjectPtr<IOpenGLTexture2D> _offscreen_depth_rt1;
-    ObjectPtr<IOpenGLTexture2D> _offscreen_color_rt2;
-    ObjectPtr<IOpenGLTexture2D> _offscreen_depth_rt2;
-    ObjectPtr<IOpenGLTexture2D> _offscreen_color_rt3;
-    ObjectPtr<IOpenGLTexture2D> _offscreen_depth_rt3;
+    ObjectPtr<IOpenGLBaseTexture> _offscreen_color_rt0;
+    ObjectPtr<IOpenGLBaseTexture> _offscreen_depth_rt0;
+    ObjectPtr<IOpenGLBaseTexture> _offscreen_color_rt1;
+    ObjectPtr<IOpenGLBaseTexture> _offscreen_depth_rt1;
+    ObjectPtr<IOpenGLBaseTexture> _offscreen_color_rt2;
+    ObjectPtr<IOpenGLBaseTexture> _offscreen_depth_rt2;
+    ObjectPtr<IOpenGLBaseTexture> _offscreen_color_rt3;
+    ObjectPtr<IOpenGLBaseTexture> _offscreen_depth_rt3;
 
     Matrix4 _projection_matrix;
     Matrix4 _model_view_matrix;
