@@ -19,38 +19,30 @@
  *
  */
 
-
+#include "GLResource.h"
+#include "GraphicsDisplay.h"
 #include "GpuDevice.h"
 #include "GLDeviceObjects.h"
 #include "IOpenGLGLSLShader.h"
 
 namespace nux
 {
+  namespace local
+  {
+  namespace
+  {
+    GLuint last_loaded_shader = 0;
+    bool enable_tracking = false;
+  }
+  }
 
   NUX_IMPLEMENT_OBJECT_TYPE (IOpenGLShader);
   NUX_IMPLEMENT_OBJECT_TYPE (IOpenGLVertexShader);
   NUX_IMPLEMENT_OBJECT_TYPE (IOpenGLPixelShader);
+  //NUX_IMPLEMENT_OBJECT_TYPE (IOpenGLGeometryShader);
   NUX_IMPLEMENT_OBJECT_TYPE (IOpenGLShaderProgram);
 
-  struct ShaderDefinition
-  {
-    NString	    Name;
-    NString     Value;
-  };
 
-// //-----------------------------------------------------------------------------
-// static void AddShaderDefinition(std::vector<ShaderDefinition>& Definitions,const TCHAR* Name,const TCHAR* Format,...)
-// {
-//     TCHAR	DefinitionText[1024];
-//     GET_VARARGS(DefinitionText, NUX_ARRAY_COUNT(DefinitionText), NUX_ARRAY_COUNT(DefinitionText)-1,Format);
-//
-//     ShaderDefinition	Definition;
-//     Definition.Name = Name;
-//     Definition.Value = DefinitionText;
-//     Definitions.push_back(Definition);
-// }
-
-//-----------------------------------------------------------------------------
   bool ExtractShaderString3 (const NString &ShaderToken, const NString &ShaderSource, NString &RetSource, NString ShaderPreprocessorDefines)
   {
     t_size lineStart = 0;
@@ -243,7 +235,7 @@ namespace nux
     Memcpy (ShaderSource, _ShaderCode.GetTCharPtr(), CodeSize);
 
     CHECKGL ( glShaderSource (_OpenGLID, 1, (const GLcharARB **) &ShaderSource, NULL) );
-    delete ShaderSource;
+    delete [] ShaderSource;
 
     // compile vertex shader object
     CHECKGL ( glCompileShader (_OpenGLID) );
@@ -269,7 +261,7 @@ namespace nux
         nuxError (TEXT ("[IOpenGLVertexShader::Compile] glCompileShader: %s"), InfoLogBuffer);
       }
 
-      delete InfoLogBuffer;
+      delete[] InfoLogBuffer;
     }
 
     return (m_CompiledAndReady ? true : false);
@@ -322,7 +314,7 @@ namespace nux
     Memset (ShaderSource, 0, CodeSize + 1);
     Memcpy (ShaderSource, _ShaderCode.m_string.c_str(), CodeSize);
     CHECKGL ( glShaderSource (_OpenGLID, 1, (const GLcharARB **) &ShaderSource, &CodeSize) );
-    delete ShaderSource;
+    delete [] ShaderSource;
 
     // compile pixel shader object
     CHECKGL ( glCompileShader (_OpenGLID) );
@@ -348,7 +340,7 @@ namespace nux
         nuxError (TEXT ("[IOpenGLPixelShader::Compile] glCompileShader: %s"), InfoLogBuffer);
       }
 
-      delete InfoLogBuffer;
+      delete[] InfoLogBuffer;
     }
 
     return (m_CompiledAndReady ? true : false);
@@ -359,106 +351,220 @@ namespace nux
     return (m_CompiledAndReady ? true : false);
   }
 
-  IOpenGLShaderProgram::IOpenGLShaderProgram (NString ShaderProgramName)
-    :   IOpenGLResource (RT_GLSL_SHADERPROGRAM)
-    ,   _FirstParameter (0)
+#if 0
+  IOpenGLGeometryShader::IOpenGLGeometryShader (NString ShaderName)
+    :   IOpenGLShader (ShaderName, RT_GLSL_GEOMETRYSHADER)
     ,   m_CompiledAndReady (false)
-    ,   _ShaderProgramName (ShaderProgramName)
+
   {
-    _OpenGLID = glCreateProgram();
-    CHECKGL_MSG ( glCreateProgram() );
+    _OpenGLID = glCreateShader(GL_GEOMETRY_SHADER);
+    CHECKGL_MSG(glCreateShader(GL_GEOMETRY_SHADER));
   }
 
-  IOpenGLShaderProgram::~IOpenGLShaderProgram()
+  IOpenGLGeometryShader::~IOpenGLGeometryShader()
   {
-    CHECKGL ( glDeleteProgram (_OpenGLID) );
+    CHECKGL(glDeleteShader(_OpenGLID));
     _OpenGLID = 0;
     m_CompiledAndReady = false;
   }
 
-  void IOpenGLShaderProgram::LoadIShaderFile (const TCHAR *ShaderFileName, const TCHAR *VtxShaderPreprocessorDefines, const TCHAR *FrgShaderPreprocessorDefines)
+  void IOpenGLGeometryShader::SetShaderCode (const TCHAR *ShaderCode, const TCHAR *GeometryShaderPreprocessorDefines)
   {
-    nuxAssertMsg (ShaderFileName, TEXT ("[IOpenGLShaderProgram::LoadIShaderFile] Invalid shader file name.") );
-    NUX_RETURN_IF_NULL (ShaderFileName);
-    NString SourceCode;
-    LoadFileToString (SourceCode, ShaderFileName);
-    LoadIShader (&SourceCode[0], VtxShaderPreprocessorDefines, FrgShaderPreprocessorDefines);
+    nuxAssertMsg (ShaderCode, TEXT("[IOpenGLGeometryShader::SetShaderCode] Invalid shader code.") );
+    NUX_RETURN_IF_NULL(ShaderCode);
+    NString ProcessedShaderSource;
+    NString Defines(GeometryShaderPreprocessorDefines);
+    InsertPreProcessorDefinitions(ShaderCode, ProcessedShaderSource, Defines);
+
+    m_CompiledAndReady = false;
+    _ShaderCode = ProcessedShaderSource;
   }
 
-  void IOpenGLShaderProgram::LoadIShader (const TCHAR *ShaderCode, const TCHAR *VtxShaderPreprocessorDefines, const TCHAR *FrgShaderPreprocessorDefines)
+  bool IOpenGLGeometryShader::Compile()
   {
-    nuxAssertMsg (ShaderCode, TEXT ("[IOpenGLShaderProgram::LoadIShader] Invalid shader code.") );
-    NUX_RETURN_IF_NULL (ShaderCode);
+
+    GLint CodeSize = (GLint) _ShaderCode.Size();
+
+    if (CodeSize == 0)
+    {
+      nuxDebugMsg(TEXT("[IOpenGLGeometryShader::Compile] Pixel shader source code is empty.") );
+    }
+
+    char *ShaderSource = new char[CodeSize+1];
+    Memset(ShaderSource, 0, CodeSize + 1);
+    Memcpy(ShaderSource, _ShaderCode.m_string.c_str(), CodeSize);
+    CHECKGL( glShaderSource(_OpenGLID, 1, (const GLcharARB **) &ShaderSource, &CodeSize) );
+    delete [] ShaderSource;
+
+    // compile pixel shader object
+    CHECKGL(glCompileShader(_OpenGLID) );
+
+    // check if shader compiled
+    m_CompiledAndReady = false;
+    CHECKGL( glGetShaderiv(_OpenGLID, GL_COMPILE_STATUS, &m_CompiledAndReady) );
+
+    if (!m_CompiledAndReady)
+    {
+      ANSICHAR *InfoLogBuffer = 0;
+      GLint InfoLogBufferSize = 0;
+      GLint InfoLogReturnSize = 0;
+      GLint iLog = 0;
+
+      glGetShaderiv(_OpenGLID, GL_INFO_LOG_LENGTH, &iLog);
+      InfoLogBuffer = new ANSICHAR[iLog+1];
+      InfoLogBufferSize = iLog + 1;
+      glGetShaderInfoLog(_OpenGLID, InfoLogBufferSize, &InfoLogReturnSize, InfoLogBuffer);
+
+      if (InfoLogReturnSize != 0)
+      {
+        nuxError(TEXT("[IOpenGLGeometryShader::Compile] glCompileShader: %s"), InfoLogBuffer);
+      }
+
+      delete InfoLogBuffer;
+    }
+
+    return (m_CompiledAndReady ? true : false);
+  }
+
+  bool IOpenGLGeometryShader::IsValid()
+  {
+    return (m_CompiledAndReady ? true : false);
+  }
+
+  void IOpenGLGeometryShader::SetInputPrimitiveType(GLenum type)
+  {
+    CHECKGL(glProgramParameteri(_OpenGLID, GL_GEOMETRY_INPUT_TYPE_EXT, type));
+  }
+
+  void IOpenGLGeometryShader::SetOutputPrimitiveType(GLenum type)
+  {
+    CHECKGL(glProgramParameteri(_OpenGLID, GL_GEOMETRY_OUTPUT_TYPE_EXT, type));
+  }
+
+  void IOpenGLGeometryShader::SetMaxVertexOutput(int max_vertex_output)
+  {
+    CHECKGL(glProgramParameteri(_OpenGLID, GL_GEOMETRY_VERTICES_OUT_EXT, max_vertex_output));
+  }
+#endif
+
+  IOpenGLShaderProgram::IOpenGLShaderProgram(NString ShaderProgramName)
+    :   IOpenGLResource(RT_GLSL_SHADERPROGRAM)
+    ,   _FirstParameter(0)
+    ,   m_CompiledAndReady(false)
+    ,   _ShaderProgramName(ShaderProgramName)
+  {
+    _OpenGLID = glCreateProgram();
+    CHECKGL_MSG( glCreateProgram() );
+  }
+
+  IOpenGLShaderProgram::~IOpenGLShaderProgram()
+  {
+    if (local::last_loaded_shader == _OpenGLID)
+    {
+      CHECKGL( glUseProgramObjectARB(0) );
+      local::last_loaded_shader = 0;
+    }
+      
+    CHECKGL( glDeleteProgram(_OpenGLID) );
+    _OpenGLID = 0;
+    m_CompiledAndReady = false;
+  }
+
+  void IOpenGLShaderProgram::LoadIShaderFile(const TCHAR *ShaderFileName, const TCHAR *VtxShaderPreprocessorDefines, const TCHAR *FrgShaderPreprocessorDefines)
+  {
+    nuxAssertMsg(ShaderFileName, TEXT("[IOpenGLShaderProgram::LoadIShaderFile] Invalid shader file name.") );
+    NUX_RETURN_IF_NULL(ShaderFileName);
+    NString SourceCode;
+    LoadFileToString(SourceCode, ShaderFileName);
+    LoadIShader(&SourceCode[0], VtxShaderPreprocessorDefines, FrgShaderPreprocessorDefines);
+  }
+
+  void IOpenGLShaderProgram::LoadIShader(const TCHAR *ShaderCode, const TCHAR *VtxShaderPreprocessorDefines, const TCHAR *FrgShaderPreprocessorDefines)
+  {
+    nuxAssertMsg(ShaderCode, TEXT("[IOpenGLShaderProgram::LoadIShader] Invalid shader code.") );
+    NUX_RETURN_IF_NULL(ShaderCode);
     NString VertexShaderSource;
-    ExtractShaderString3 (TEXT ("[Vertex Shader]"), ShaderCode, VertexShaderSource, NString (VtxShaderPreprocessorDefines) );
+    ExtractShaderString3(TEXT("[Vertex Shader]"), ShaderCode, VertexShaderSource, NString(VtxShaderPreprocessorDefines) );
     NString PixelShaderSource;
-    ExtractShaderString3 (TEXT ("[Fragment Shader]"), ShaderCode, PixelShaderSource, NString (FrgShaderPreprocessorDefines) );
+    ExtractShaderString3(TEXT("[Fragment Shader]"), ShaderCode, PixelShaderSource, NString(FrgShaderPreprocessorDefines) );
 
-    ObjectPtr<IOpenGLVertexShader> vs = GetGpuDevice()->CreateVertexShader(); //new IOpenGLVertexShader;
-    ObjectPtr<IOpenGLPixelShader> ps = GetGpuDevice()->CreatePixelShader(); //new IOpenGLPixelShader;
+    ObjectPtr<IOpenGLVertexShader> vs = GetGraphicsDisplay()->GetGpuDevice()->CreateVertexShader(); //new IOpenGLVertexShader;
+    ObjectPtr<IOpenGLPixelShader> ps = GetGraphicsDisplay()->GetGpuDevice()->CreatePixelShader(); //new IOpenGLPixelShader;
 
-    vs->SetShaderCode (&VertexShaderSource[0]);
-    ps->SetShaderCode (&PixelShaderSource[0]);
+    vs->SetShaderCode(&VertexShaderSource[0]);
+    ps->SetShaderCode(&PixelShaderSource[0]);
     vs->Compile();
     ps->Compile();
 
     ShaderObjectList.clear();
 
-    AddShaderObject (vs);
-    AddShaderObject (ps);
+    AddShaderObject(vs);
+    AddShaderObject(ps);
   }
 
-  void IOpenGLShaderProgram::LoadVertexShader (const TCHAR *glslshader, const TCHAR *VtxShaderPreprocessorDefines)
+  void IOpenGLShaderProgram::LoadVertexShader(const TCHAR *glslshader, const TCHAR *VtxShaderPreprocessorDefines)
   {
-    nuxAssertMsg (glslshader, TEXT ("[IOpenGLShaderProgram::LoadVertexShader] Invalid shader code.") );
-    NUX_RETURN_IF_NULL (glslshader);
-    ObjectPtr<IOpenGLVertexShader> vs = GetGpuDevice()->CreateVertexShader(); //new IOpenGLVertexShader;
+    nuxAssertMsg(glslshader, TEXT("[IOpenGLShaderProgram::LoadVertexShader] Invalid shader code.") );
+    NUX_RETURN_IF_NULL(glslshader);
+    ObjectPtr<IOpenGLVertexShader> vs = GetGraphicsDisplay()->GetGpuDevice()->CreateVertexShader(); //new IOpenGLVertexShader;
 
     NString ProcessedShaderSource;
-    NString Defines (VtxShaderPreprocessorDefines);
-    InsertPreProcessorDefinitions (glslshader, ProcessedShaderSource, Defines);
+    NString Defines(VtxShaderPreprocessorDefines);
+    InsertPreProcessorDefinitions(glslshader, ProcessedShaderSource, Defines);
 
-    vs->SetShaderCode (glslshader);
+    vs->SetShaderCode(glslshader);
     vs->Compile();
-    AddShaderObject (vs);
+    AddShaderObject(vs);
   }
 
-  void IOpenGLShaderProgram::LoadPixelShader (const TCHAR *glslshader, const TCHAR *FrgShaderPreprocessorDefines)
+  void IOpenGLShaderProgram::LoadPixelShader(const TCHAR *glslshader, const TCHAR *FrgShaderPreprocessorDefines)
   {
-    nuxAssertMsg (glslshader, TEXT ("[IOpenGLShaderProgram::LoadPixelShader] Invalid shader code.") );
-    NUX_RETURN_IF_NULL (glslshader);
-    ObjectPtr<IOpenGLPixelShader> ps = GetGpuDevice()->CreatePixelShader(); //new IOpenGLPixelShader;
+    nuxAssertMsg(glslshader, TEXT("[IOpenGLShaderProgram::LoadPixelShader] Invalid shader code.") );
+    NUX_RETURN_IF_NULL(glslshader);
+    ObjectPtr<IOpenGLPixelShader> ps = GetGraphicsDisplay()->GetGpuDevice()->CreatePixelShader(); //new IOpenGLPixelShader;
 
     NString ProcessedShaderSource;
-    NString Defines (FrgShaderPreprocessorDefines);
-    InsertPreProcessorDefinitions (glslshader, ProcessedShaderSource, Defines);
+    NString Defines(FrgShaderPreprocessorDefines);
+    InsertPreProcessorDefinitions(glslshader, ProcessedShaderSource, Defines);
 
-    ps->SetShaderCode (glslshader);
+    ps->SetShaderCode(glslshader);
     ps->Compile();
-    AddShaderObject (ps);
+    AddShaderObject(ps);
   }
 
-  void IOpenGLShaderProgram::AddShaderObject (ObjectPtr<IOpenGLShader> ShaderObject)
+  void IOpenGLShaderProgram::AddShaderObject(ObjectPtr<IOpenGLShader> ShaderObject)
   {
-    ShaderObjectList.push_back (ShaderObject);
+    ShaderObjectList.push_back(ShaderObject);
   }
 
-  void IOpenGLShaderProgram::AddShaderParameter (GLShaderParameter *Parameter)
+  void IOpenGLShaderProgram::AddShaderParameter(GLShaderParameter* parameter)
   {
-    Parameter->m_NextParameter = _FirstParameter;
-    _FirstParameter = Parameter;
+    GLShaderParameter* temp = _FirstParameter;
+
+    while(temp)
+    {
+      if(temp == parameter)
+      {
+        // Parameter already added
+        return;
+      }
+      temp = temp->m_NextParameter;
+    }
+
+    parameter->m_NextParameter = _FirstParameter;
+    _FirstParameter = parameter;
 
     // If we add shader parameters after the program is linked, we need to call CheckUniformLocation().
     CheckUniformLocation();
   }
 
-  void IOpenGLShaderProgram::RemoveShaderObject (ObjectPtr<IOpenGLShader> ShaderObject)
+  void IOpenGLShaderProgram::RemoveShaderObject(ObjectPtr<IOpenGLShader> ShaderObject)
   {
-    std::vector< ObjectPtr<IOpenGLShader> >::iterator it = find (ShaderObjectList.begin(), ShaderObjectList.end(), ShaderObject);
+    std::vector< ObjectPtr<IOpenGLShader> >::iterator it = find(ShaderObjectList.begin(), ShaderObjectList.end(), ShaderObject);
 
     if (it != ShaderObjectList.end() )
     {
-      ShaderObjectList.erase (it);
+      ShaderObjectList.erase(it);
     }
   }
 
@@ -471,7 +577,7 @@ namespace nux
   {
     // Get the number of attached shaders.
     GLint NumAttachedShaders;
-    CHECKGL ( glGetProgramiv (_OpenGLID, GL_ATTACHED_SHADERS, &NumAttachedShaders) );
+    CHECKGL( glGetProgramiv(_OpenGLID, GL_ATTACHED_SHADERS, &NumAttachedShaders) );
     GLuint *ShaderObjects = 0;
 
     if (NumAttachedShaders)
@@ -479,18 +585,18 @@ namespace nux
       ShaderObjects = new GLuint[NumAttachedShaders];
     }
 
-    CHECKGL ( glGetAttachedShaders (_OpenGLID, NumAttachedShaders, NULL, ShaderObjects) );
+    CHECKGL( glGetAttachedShaders(_OpenGLID, NumAttachedShaders, NULL, ShaderObjects) );
 
     // Detach everything first
     for (int i = 0; i < (int) NumAttachedShaders; i++)
     {
       unsigned int obj = ShaderObjects[i];
-      CHECKGL ( glDetachShader (_OpenGLID, obj) );
+      CHECKGL( glDetachShader(_OpenGLID, obj) );
     }
 
     if (NumAttachedShaders)
     {
-      delete ShaderObjects;
+      delete[] ShaderObjects;
     }
 
     for (int i = 0; i < (int) ShaderObjectList.size(); i++)
@@ -499,17 +605,17 @@ namespace nux
       {
         if (ShaderObjectList[i]->Compile() == false)
         {
-          nuxDebugMsg (TEXT ("[IOpenGLShaderProgram::Link] Attached shader %s does not compile with program: %s."), ShaderObjectList[i]->_ShaderName.GetTCharPtr(), _ShaderProgramName.GetTCharPtr() );
+          nuxDebugMsg(TEXT("[IOpenGLShaderProgram::Link] Attached shader %s does not compile with program: %s."), ShaderObjectList[i]->_ShaderName.GetTCharPtr(), _ShaderProgramName.GetTCharPtr() );
         }
       }
 
       unsigned int obj = ShaderObjectList[i]->GetOpenGLID();
-      CHECKGL ( glAttachShader (_OpenGLID, obj) );
+      CHECKGL( glAttachShader(_OpenGLID, obj) );
     }
 
     GLint linked;
-    CHECKGL ( glLinkProgram (_OpenGLID) );
-    CHECKGL ( glGetProgramiv (_OpenGLID, GL_LINK_STATUS, &linked) );
+    CHECKGL( glLinkProgram(_OpenGLID) );
+    CHECKGL( glGetProgramiv(_OpenGLID, GL_LINK_STATUS, &linked) );
 
     if (linked == GL_FALSE)
     {
@@ -517,26 +623,26 @@ namespace nux
       GLint InfoLogBufferSize = 0;
       GLint InfoLogReturnSize = 0;
       GLint iLog = 0;
-      glGetProgramiv (_OpenGLID, GL_INFO_LOG_LENGTH, &iLog);
+      glGetProgramiv(_OpenGLID, GL_INFO_LOG_LENGTH, &iLog);
       InfoLogBuffer = new ANSICHAR[iLog+1];
       InfoLogBufferSize = iLog + 1;
 
-      glGetProgramInfoLog (_OpenGLID, InfoLogBufferSize, &InfoLogReturnSize, InfoLogBuffer);
+      glGetProgramInfoLog(_OpenGLID, InfoLogBufferSize, &InfoLogReturnSize, InfoLogBuffer);
 
       if (InfoLogReturnSize != 0)
       {
-        nuxError (TEXT ("[IOpenGLShaderProgram::Link] glLinkProgram: %s"), InfoLogBuffer);
+        nuxError(TEXT("[IOpenGLShaderProgram::Link] glLinkProgram: %s"), InfoLogBuffer);
       }
 
-      delete InfoLogBuffer;
+      delete[] InfoLogBuffer;
       m_CompiledAndReady = false;
       return m_CompiledAndReady;
     }
 
     GLint validated;
     // glValidateProgram checks to see whether the executables contained in program can execute given the current OpenGL state.
-    CHECKGL ( glValidateProgram (_OpenGLID) );
-    CHECKGL ( glGetProgramiv (_OpenGLID, GL_VALIDATE_STATUS, &validated) );
+    CHECKGL( glValidateProgram(_OpenGLID) );
+    CHECKGL( glGetProgramiv(_OpenGLID, GL_VALIDATE_STATUS, &validated) );
 
     if (validated == GL_FALSE)
     {
@@ -544,38 +650,51 @@ namespace nux
       GLint InfoLogBufferSize = 0;
       GLint InfoLogReturnSize = 0;
       GLint iLog = 0;
-      glGetProgramiv (_OpenGLID, GL_INFO_LOG_LENGTH, &iLog);
+      glGetProgramiv(_OpenGLID, GL_INFO_LOG_LENGTH, &iLog);
       InfoLogBuffer = new ANSICHAR[iLog+1];
       InfoLogBufferSize = iLog + 1;
 
-      glGetProgramInfoLog (_OpenGLID, InfoLogBufferSize, &InfoLogReturnSize, InfoLogBuffer);
+      glGetProgramInfoLog(_OpenGLID, InfoLogBufferSize, &InfoLogReturnSize, InfoLogBuffer);
 
       if (InfoLogReturnSize != 0)
       {
-        nuxError (TEXT ("[IOpenGLShaderProgram::Link] glValidateProgram: %s"), InfoLogBuffer);
+        nuxError(TEXT("[IOpenGLShaderProgram::Link] glValidateProgram: %s"), InfoLogBuffer);
       }
 
-      delete InfoLogBuffer;
+      delete [] InfoLogBuffer;
     }
 
     m_CompiledAndReady = true;
 
     Begin();
-    CheckAttributeLocation();
     CheckUniformLocation();
+    CheckAttributeLocation();
+    
     End();
 
     return m_CompiledAndReady;
   }
 
-  void IOpenGLShaderProgram::Begin (void)
+  void IOpenGLShaderProgram::SetShaderTracking (bool enabled)
   {
-    CHECKGL ( glUseProgramObjectARB (_OpenGLID) );
+    local::enable_tracking = enabled;
+    local::last_loaded_shader = 0;
+    CHECKGL( glUseProgramObjectARB(0) );
   }
 
-  void IOpenGLShaderProgram::End (void)
+  void IOpenGLShaderProgram::Begin(void)
   {
-    CHECKGL ( glUseProgramObjectARB (0) );
+    if (local::last_loaded_shader == _OpenGLID && local::enable_tracking)
+      return;
+    
+    local::last_loaded_shader = _OpenGLID;
+    CHECKGL( glUseProgramObjectARB(_OpenGLID) );
+  }
+
+  void IOpenGLShaderProgram::End(void)
+  {
+    if (!local::enable_tracking)
+      CHECKGL( glUseProgramObjectARB(0) );
   }
 
   void IOpenGLShaderProgram::CheckAttributeLocation()
@@ -595,7 +714,7 @@ namespace nux
     GLenum type;
 
     GLint num_active_attributes;
-    CHECKGL ( glGetObjectParameterivARB (_OpenGLID, GL_OBJECT_ACTIVE_ATTRIBUTES_ARB, &num_active_attributes) );
+    CHECKGL( glGetObjectParameterivARB(_OpenGLID, GL_OBJECT_ACTIVE_ATTRIBUTES_ARB, &num_active_attributes) );
 
 
     //         Vertex Attribute Aliasing
@@ -636,7 +755,8 @@ namespace nux
                             &type,
                             active_attribute_name);
       CHECKGL_MSG ( glGetActiveAttribARB );
-      m_ProgramAttributeDefinition[index].attribute_index = index;
+      m_ProgramAttributeDefinition[index].attribute_index = glGetAttribLocationARB(_OpenGLID, active_attribute_name);
+      CHECKGL_MSG ( glGetAttribLocationARB );
       m_ProgramAttributeDefinition[index].attribute_name = active_attribute_name;
       m_ProgramAttributeDefinition[index].valid = true;
 
@@ -660,58 +780,9 @@ namespace nux
         case GL_FLOAT_MAT4:
         default:
           //todo
-          nuxAssert (0);
+          nuxAssert(0);
       }
     }
-
-
-    //    if(sad == 0)
-    //        return;
-    //    int n = sizeof(sad) / sizeof(IOpenGLShaderAttributeDefinition);
-    //    for(int i = 0; i < num_active_attributes; i++)
-    //    {
-    //        bool found = false;
-    //        for(int j = 0; j < n; j++)
-    //        {
-    //            if(m_ProgramAttributeDefinition[i].attribute_name == sad[j].attribute_name)
-    //            {
-    //                found = true;
-    //                if(m_ProgramAttributeDefinition[i].attribute_index != sad[j].attribute_index)
-    //                    OutputDebugString("*** Active attribute has incorrect index.\n");
-    //                if(m_ProgramAttributeDefinition[i].type != sad[j].type)
-    //                    OutputDebugString("*** Active attribute has incorrect type.\n");
-    //                break;
-    //            }
-    //            if(found == false)
-    //            {
-    //                OutputDebugString("*** Active binded but not requested in meta shader info.\n");
-    //                nuxAssert(0);
-    //            }
-    //        }
-    //    }
-    //
-    //
-    //    for(int i = 0; i < n; i++)
-    //    {
-    //        bool found = false;
-    //        for(int j = 0; j < num_active_attributes; j++)
-    //        {
-    //            if(sad[i].attribute_name == m_ProgramAttributeDefinition[j].attribute_name)
-    //            {
-    //                found = true;
-    //                if(sad[i].attribute_index != m_ProgramAttributeDefinition[j].attribute_index)
-    //                    OutputDebugString("*** Active attribute has incorrect index.\n");
-    //                if(sad[i].type != m_ProgramAttributeDefinition[j].type)
-    //                    OutputDebugString("*** Active attribute has incorrect type.\n");
-    //                break;
-    //            }
-    //            if(found == false)
-    //            {
-    //                OutputDebugString("*** Active requested in meta shader info but not binded.\n");
-    //                nuxAssert(0);
-    //            }
-    //        }
-    //    }
   }
 
   void IOpenGLShaderProgram::CheckUniformLocation()
@@ -730,14 +801,11 @@ namespace nux
         nuxAssert (0);
       }
 
-      //GLsizei length;
       GLint size = 0;
       GLenum type = 0;
 
       if (location >= 0)
       {
-        //nuxDebugMsg(TEXT("[IOpenGLShaderProgram::CheckUniformLocation] Program OpenGL ID: %d"), _OpenGLID);
-        //nuxDebugMsg(TEXT("[IOpenGLShaderProgram::CheckUniformLocation] Location index:%s %d"), TCHAR_TO_ANSI(parameter->m_Name.GetTCharPtr()), location);
         CHECKGL ( glGetActiveUniformARB (_OpenGLID, location, 0, NULL /*&length*/, &size, &type, NULL) );
       }
 
@@ -760,12 +828,8 @@ namespace nux
     return -1;
   }
 
-//-----------------------------------------------------------------------------
   bool IOpenGLShaderProgram::SetUniform1f (char *varname, GLfloat v0)
   {
-    //if (!useGLSL) return false; // GLSL not available
-    //if (!_noshader) return true;
-
     GLint loc = GetUniformLocationARB (varname);
 
     if (loc == -1) return false; // can't find variable
@@ -776,22 +840,15 @@ namespace nux
   }
   bool IOpenGLShaderProgram::SetUniform1f (GLint loc, GLfloat v0)
   {
-    //if (!useGLSL) return false; // GLSL not available
-    //if (!_noshader) return true;
-
     if (loc == -1) return false; // can't find variable
 
     glUniform1fARB (loc, v0);
     return true;
   }
 
-//-----------------------------------------------------------------------------
 
   bool IOpenGLShaderProgram::SetUniform2f (char *varname, GLfloat v0, GLfloat v1)
   {
-    //if (!useGLSL) return false; // GLSL not available
-    //if (!_noshader) return true;
-
     GLint loc = GetUniformLocationARB (varname);
 
     if (loc == -1) return false; // can't find variable
@@ -802,21 +859,14 @@ namespace nux
   }
   bool IOpenGLShaderProgram::SetUniform2f (GLint loc, GLfloat v0, GLfloat v1)
   {
-    //if (!useGLSL) return false; // GLSL not available
-    //if (!_noshader) return true;
-
     if (loc == -1) return false; // can't find variable
 
     glUniform2fARB (loc, v0, v1);
     return true;
   }
-//-----------------------------------------------------------------------------
 
   bool IOpenGLShaderProgram::SetUniform3f (char *varname, GLfloat v0, GLfloat v1, GLfloat v2)
   {
-    //if (!useGLSL) return false; // GLSL not available
-    //if (!_noshader) return true;
-
     GLint loc = GetUniformLocationARB (varname);
 
     if (loc == -1) return false; // can't find variable
@@ -827,22 +877,15 @@ namespace nux
   }
   bool IOpenGLShaderProgram::SetUniform3f (GLint loc, GLfloat v0, GLfloat v1, GLfloat v2)
   {
-    //if (!useGLSL) return false; // GLSL not available
-    //if (!_noshader) return true;
-
     if (loc == -1) return false; // can't find variable
 
     glUniform3fARB (loc, v0, v1, v2);
 
     return true;
   }
-//-----------------------------------------------------------------------------
 
   bool IOpenGLShaderProgram::SetUniform4f (char *varname, GLfloat v0, GLfloat v1, GLfloat v2, GLfloat v3)
   {
-    //if (!useGLSL) return false; // GLSL not available
-    //if (!_noshader) return true;
-
     GLint loc = GetUniformLocationARB (varname);
 
     if (loc == -1) return false; // can't find variable
@@ -853,22 +896,15 @@ namespace nux
   }
   bool IOpenGLShaderProgram::SetUniform4f (GLint loc, GLfloat v0, GLfloat v1, GLfloat v2, GLfloat v3)
   {
-    //if (!useGLSL) return false; // GLSL not available
-    //if (!_noshader) return true;
-
     if (loc == -1) return false; // can't find variable
 
     glUniform4fARB (loc, v0, v1, v2, v3);
 
     return true;
   }
-//-----------------------------------------------------------------------------
 
   bool IOpenGLShaderProgram::SetUniform1i (char *varname, GLint v0)
   {
-    //if (!useGLSL) return false; // GLSL not available
-    //if (!_noshader) return true;
-
     GLint loc = GetUniformLocationARB (varname);
 
     if (loc == -1) return false; // can't find variable
@@ -879,9 +915,6 @@ namespace nux
   }
   bool IOpenGLShaderProgram::SetUniform1i (GLint loc, GLint v0)
   {
-    //if (!useGLSL) return false; // GLSL not available
-    //if (!_noshader) return true;
-
     if (loc == -1) return false; // can't find variable
 
     glUniform1iARB (loc, v0);
@@ -891,9 +924,6 @@ namespace nux
 
   bool IOpenGLShaderProgram::SetUniform2i (char *varname, GLint v0, GLint v1)
   {
-    //    if (!useGLSL) return false; // GLSL not available
-    //    if (!_noshader) return true;
-
     GLint loc = GetUniformLocationARB (varname);
 
     if (loc == -1) return false; // can't find variable
@@ -905,9 +935,6 @@ namespace nux
   }
   bool IOpenGLShaderProgram::SetUniform2i (GLint loc, GLint v0, GLint v1)
   {
-    //    if (!useGLSL) return false; // GLSL not available
-    //    if (!_noshader) return true;
-
     if (loc == -1) return false; // can't find variable
 
     glUniform2iARB (loc, v0, v1);
@@ -915,13 +942,9 @@ namespace nux
 
     return true;
   }
-//-----------------------------------------------------------------------------
 
   bool IOpenGLShaderProgram::SetUniform3i (char *varname, GLint v0, GLint v1, GLint v2)
   {
-    //    if (!useGLSL) return false; // GLSL not available
-    //    if (!_noshader) return true;
-
     GLint loc = GetUniformLocationARB (varname);
 
     if (loc == -1) return false; // can't find variable
@@ -932,9 +955,6 @@ namespace nux
   }
   bool IOpenGLShaderProgram::SetUniform3i (GLint loc, GLint v0, GLint v1, GLint v2)
   {
-    //    if (!useGLSL) return false; // GLSL not available
-    //    if (!_noshader) return true;
-
     if (loc == -1) return false; // can't find variable
 
     glUniform3iARB (loc, v0, v1, v2);
@@ -944,9 +964,6 @@ namespace nux
 
   bool IOpenGLShaderProgram::SetUniform4i (char *varname, GLint v0, GLint v1, GLint v2, GLint v3)
   {
-    //    if (!useGLSL) return false; // GLSL not available
-    //    if (!_noshader) return true;
-
     GLint loc = GetUniformLocationARB (varname);
 
     if (loc == -1) return false; // can't find variable
@@ -957,22 +974,15 @@ namespace nux
   }
   bool IOpenGLShaderProgram::SetUniform4i (GLint loc, GLint v0, GLint v1, GLint v2, GLint v3)
   {
-    //    if (!useGLSL) return false; // GLSL not available
-    //    if (!_noshader) return true;
-
     if (loc == -1) return false; // can't find variable
 
     glUniform4iARB (loc, v0, v1, v2, v3);
 
     return true;
   }
-//-----------------------------------------------------------------------------
 
   bool IOpenGLShaderProgram::SetUniform1fv (char *varname, GLsizei count, GLfloat *value)
   {
-    //    if (!useGLSL) return false; // GLSL not available
-    //    if (!_noshader) return true;
-
     GLint loc = GetUniformLocationARB (varname);
 
     if (loc == -1) return false; // can't find variable
@@ -983,9 +993,6 @@ namespace nux
   }
   bool IOpenGLShaderProgram::SetUniform1fv (GLint loc, GLsizei count, GLfloat *value)
   {
-    //    if (!useGLSL) return false; // GLSL not available
-    //    if (!_noshader) return true;
-
     if (loc == -1) return false; // can't find variable
 
     glUniform1fvARB (loc, count, value);
@@ -995,9 +1002,6 @@ namespace nux
 
   bool IOpenGLShaderProgram::SetUniform2fv (char *varname, GLsizei count, GLfloat *value)
   {
-    //    if (!useGLSL) return false; // GLSL not available
-    //    if (!_noshader) return true;
-
     GLint loc = GetUniformLocationARB (varname);
 
     if (loc == -1) return false; // can't find variable
@@ -1008,22 +1012,15 @@ namespace nux
   }
   bool IOpenGLShaderProgram::SetUniform2fv (GLint loc, GLsizei count, GLfloat *value)
   {
-    //    if (!useGLSL) return false; // GLSL not available
-    //    if (!_noshader) return true;
-
     if (loc == -1) return false; // can't find variable
 
     glUniform2fvARB (loc, count, value);
 
     return true;
   }
-//-----------------------------------------------------------------------------
 
   bool IOpenGLShaderProgram::SetUniform3fv (char *varname, GLsizei count, GLfloat *value)
   {
-    //    if (!useGLSL) return false; // GLSL not available
-    //    if (!_noshader) return true;
-
     GLint loc = GetUniformLocationARB (varname);
 
     if (loc == -1) return false; // can't find variable
@@ -1034,22 +1031,15 @@ namespace nux
   }
   bool IOpenGLShaderProgram::SetUniform3fv (GLint loc, GLsizei count, GLfloat *value)
   {
-    //    if (!useGLSL) return false; // GLSL not available
-    //    if (!_noshader) return true;
-
     if (loc == -1) return false; // can't find variable
 
     glUniform3fvARB (loc, count, value);
 
     return true;
   }
-//-----------------------------------------------------------------------------
 
   bool IOpenGLShaderProgram::SetUniform4fv (char *varname, GLsizei count, GLfloat *value)
   {
-    //    if (!useGLSL) return false; // GLSL not available
-    //    if (!_noshader) return true;
-
     GLint loc = GetUniformLocationARB (varname);
 
     if (loc == -1) return false; // can't find variable
@@ -1060,22 +1050,15 @@ namespace nux
   }
   bool IOpenGLShaderProgram::SetUniform4fv (GLint loc, GLsizei count, GLfloat *value)
   {
-    //    if (!useGLSL) return false; // GLSL not available
-    //    if (!_noshader) return true;
-
     if (loc == -1) return false; // can't find variable
 
     glUniform4fvARB (loc, count, value);
 
     return true;
   }
-//-----------------------------------------------------------------------------
 
   bool IOpenGLShaderProgram::SetUniform1iv (char *varname, GLsizei count, GLint *value)
   {
-    //    if (!useGLSL) return false; // GLSL not available
-    //    if (!_noshader) return true;
-
     GLint loc = GetUniformLocationARB (varname);
 
     if (loc == -1) return false; // can't find variable
@@ -1086,22 +1069,15 @@ namespace nux
   }
   bool IOpenGLShaderProgram::SetUniform1iv (GLint loc, GLsizei count, GLint *value)
   {
-    //    if (!useGLSL) return false; // GLSL not available
-    //    if (!_noshader) return true;
-
     if (loc == -1) return false; // can't find variable
 
     glUniform1ivARB (loc, count, value);
 
     return true;
   }
-//-----------------------------------------------------------------------------
 
   bool IOpenGLShaderProgram::SetUniform2iv (char *varname, GLsizei count, GLint *value)
   {
-    //    if (!useGLSL) return false; // GLSL not available
-    //    if (!_noshader) return true;
-
     GLint loc = GetUniformLocationARB (varname);
 
     if (loc == -1) return false; // can't find variable
@@ -1112,22 +1088,15 @@ namespace nux
   }
   bool IOpenGLShaderProgram::SetUniform2iv (GLint loc, GLsizei count, GLint *value)
   {
-    //    if (!useGLSL) return false; // GLSL not available
-    //    if (!_noshader) return true;
-
     if (loc == -1) return false; // can't find variable
 
     glUniform2ivARB (loc, count, value);
 
     return true;
   }
-//-----------------------------------------------------------------------------
 
   bool IOpenGLShaderProgram::SetUniform3iv (char *varname, GLsizei count, GLint *value)
   {
-    //    if (!useGLSL) return false; // GLSL not available
-    //    if (!_noshader) return true;
-
     GLint loc = GetUniformLocationARB (varname);
 
     if (loc == -1) return false; // can't find variable
@@ -1138,22 +1107,15 @@ namespace nux
   }
   bool IOpenGLShaderProgram::SetUniform3iv (GLint loc, GLsizei count, GLint *value)
   {
-    //    if (!useGLSL) return false; // GLSL not available
-    //    if (!_noshader) return true;
-
     if (loc == -1) return false; // can't find variable
 
     glUniform3ivARB (loc, count, value);
 
     return true;
   }
-//-----------------------------------------------------------------------------
 
   bool IOpenGLShaderProgram::SetUniform4iv (char *varname, GLsizei count, GLint *value)
   {
-    //    if (!useGLSL) return false; // GLSL not available
-    //    if (!_noshader) return true;
-
     GLint loc = GetUniformLocationARB (varname);
 
     if (loc == -1) return false; // can't find variable
@@ -1164,22 +1126,15 @@ namespace nux
   }
   bool IOpenGLShaderProgram::SetUniform4iv (GLint loc, GLsizei count, GLint *value)
   {
-    //    if (!useGLSL) return false; // GLSL not available
-    //    if (!_noshader) return true;
-
     if (loc == -1) return false; // can't find variable
 
     glUniform4ivARB (loc, count, value);
 
     return true;
   }
-//-----------------------------------------------------------------------------
 
   bool IOpenGLShaderProgram::SetUniformMatrix2fv (char *varname, GLsizei count, GLboolean transpose, GLfloat *value)
   {
-    //    if (!useGLSL) return false; // GLSL not available
-    //    if (!_noshader) return true;
-
     GLint loc = GetUniformLocationARB (varname);
 
     if (loc == -1) return false; // can't find variable
@@ -1190,22 +1145,15 @@ namespace nux
   }
   bool IOpenGLShaderProgram::SetUniformLocMatrix2fv (GLint loc, GLsizei count, GLboolean transpose, GLfloat *value)
   {
-    //    if (!useGLSL) return false; // GLSL not available
-    //    if (!_noshader) return true;
-
     if (loc == -1) return false; // can't find variable
 
     glUniformMatrix2fvARB (loc, count, transpose, value);
 
     return true;
   }
-//-----------------------------------------------------------------------------
 
   bool IOpenGLShaderProgram::SetUniformMatrix3fv (char *varname, GLsizei count, GLboolean transpose, GLfloat *value)
   {
-    //    if (!useGLSL) return false; // GLSL not available
-    //    if (!_noshader) return true;
-
     GLint loc = GetUniformLocationARB (varname);
 
     if (loc == -1) return false; // can't find variable
@@ -1216,22 +1164,15 @@ namespace nux
   }
   bool IOpenGLShaderProgram::SetUniformLocMatrix3fv (GLint loc, GLsizei count, GLboolean transpose, GLfloat *value)
   {
-    //    if (!useGLSL) return false; // GLSL not available
-    //    if (!_noshader) return true;
-
     if (loc == -1) return false; // can't find variable
 
     glUniformMatrix3fvARB (loc, count, transpose, value);
 
     return true;
   }
-//-----------------------------------------------------------------------------
 
   bool IOpenGLShaderProgram::SetUniformMatrix4fv (char *varname, GLsizei count, GLboolean transpose, GLfloat *value)
   {
-    //    if (!useGLSL) return false; // GLSL not available
-    //    if (!_noshader) return true;
-
     GLint loc = GetUniformLocationARB (varname);
 
     if (loc == -1) return false; // can't find variable
@@ -1242,9 +1183,6 @@ namespace nux
   }
   bool IOpenGLShaderProgram::SetUniformLocMatrix4fv (GLint loc, GLsizei count, GLboolean transpose, GLfloat *value)
   {
-    //    if (!useGLSL) return false; // GLSL not available
-    //    if (!_noshader) return true;
-
     if (loc == -1) return false; // can't find variable
 
     glUniformMatrix4fvARB (loc, count, transpose, value);
@@ -1252,11 +1190,9 @@ namespace nux
     return true;
   }
 
-//-----------------------------------------------------------------------------
 
   void IOpenGLShaderProgram::GetUniformfv (char *name, GLfloat *values)
   {
-    //     if (!useGLSL) return;
     GLint loc;
 
     loc = glGetUniformLocationARB (_OpenGLID, name);
@@ -1270,11 +1206,9 @@ namespace nux
     CHECKGL ( glGetUniformfvARB (_OpenGLID, loc, values) );
   }
 
-//-----------------------------------------------------------------------------
 
   void IOpenGLShaderProgram::GetUniformiv (char *name, GLint *values)
   {
-    //if (!useGLSL) return;
     GLint loc;
     loc = glGetUniformLocationARB (_OpenGLID, name);
     CHECKGL_MSG ( glGetUniformLocationARB );
@@ -1286,7 +1220,6 @@ namespace nux
 
     CHECKGL ( glGetUniformivARB (_OpenGLID, loc, values) );
   }
-//-----------------------------------------------------------------------------
 
   int IOpenGLShaderProgram::GetUniformLocationARB (const GLcharARB *name)
   {
@@ -1296,7 +1229,6 @@ namespace nux
     return loc;
   }
 
-//-----------------------------------------------------------------------------
   void IOpenGLShaderProgram::GetActiveUniformARB (
     GLuint index,
     GLsizei maxLength,
@@ -1316,7 +1248,6 @@ namespace nux
     CHECKGL_MSG (glGetActiveUniformARB);
   }
 
-//-----------------------------------------------------------------------------
 
   void IOpenGLShaderProgram::GetObjectParameterfvARB (GLenum pname,
       GLfloat *params)
@@ -1327,9 +1258,7 @@ namespace nux
     CHECKGL_MSG (glGetObjectParameterfvARB);
   }
 
-//-----------------------------------------------------------------------------
-
-  BOOL IOpenGLShaderProgram::SetSampler (char *name, int texture_unit)
+  bool IOpenGLShaderProgram::SetSampler (char *name, int texture_unit)
   {
     GLint loc = GetUniformLocationARB (name);
 
