@@ -21,6 +21,7 @@
 
 
 #include "GLResource.h"
+#include "GraphicsDisplay.h"
 #include "GpuDevice.h"
 #include "GLDeviceObjects.h"
 #include "IOpenGLSurface.h"
@@ -71,29 +72,12 @@ namespace nux
     unsigned int BytePerPixel = 0;
 
     IOpenGLBaseTexture *texture = _BaseTexture;
-    OpenGLResourceType ResourceType = RTBASETEXTURE;
 
-    if (_BaseTexture->_ResourceType == RTTEXTURE)
-    {
-      ResourceType = RTTEXTURE;
-    }
-    else if (_BaseTexture->_ResourceType == RTTEXTURERECTANGLE)
-    {
-      ResourceType = RTTEXTURERECTANGLE;
-    }
-    else if (_BaseTexture->_ResourceType == RTCUBETEXTURE)
-    {
-      ResourceType = RTCUBETEXTURE;
-    }
-    else if (_BaseTexture->_ResourceType == RTVOLUMETEXTURE)
-    {
-      ResourceType = RTVOLUMETEXTURE;
-    }
-    else if (_BaseTexture->_ResourceType == RTANIMATEDTEXTURE)
-    {
-      ResourceType = RTANIMATEDTEXTURE;
-    }
-    else
+    if (!((_BaseTexture->_ResourceType == RTTEXTURE) ||
+          (_BaseTexture->_ResourceType == RTTEXTURERECTANGLE) ||
+          (_BaseTexture->_ResourceType == RTCUBETEXTURE) ||
+          (_BaseTexture->_ResourceType == RTVOLUMETEXTURE) ||
+          (_BaseTexture->_ResourceType == RTANIMATEDTEXTURE)))
     {
       nuxAssertMsg (0, TEXT ("Unknown resource type") );
     }
@@ -177,17 +161,17 @@ namespace nux
     _Rect.right   = texwidth;
 
 
-    if (GetGpuDevice()->UsePixelBufferObjects() )
+    if (GetGraphicsDisplay()->GetGpuDevice()->UsePixelBufferObjects() )
     {
-      GetGpuDevice()->AllocateUnpackPixelBufferIndex (&_AllocatedUnpackBuffer);
+      GetGraphicsDisplay()->GetGpuDevice()->AllocateUnpackPixelBufferIndex (&_AllocatedUnpackBuffer);
     }
 
     if (pRect == 0)
     {
-      if (GetGpuDevice()->UsePixelBufferObjects() )
+      if (GetGraphicsDisplay()->GetGpuDevice()->UsePixelBufferObjects() )
       {
         // Mapping the entire area of the surface
-        _LockedRect.pBits = GetGpuDevice()->LockUnpackPixelBufferIndex (_AllocatedUnpackBuffer, surface_size);
+        _LockedRect.pBits = GetGraphicsDisplay()->GetGpuDevice()->LockUnpackPixelBufferIndex (_AllocatedUnpackBuffer, surface_size);
         pLockedRect->pBits = _LockedRect.pBits;
         pLockedRect->Pitch = _LockedRect.Pitch;
       }
@@ -223,9 +207,9 @@ namespace nux
         return OGL_INVALID_LOCK;
       }
 
-      if (GetGpuDevice()->UsePixelBufferObjects() )
+      if (GetGraphicsDisplay()->GetGpuDevice()->UsePixelBufferObjects() )
       {
-        _LockedRect.pBits = GetGpuDevice()->LockUnpackPixelBufferIndex (_AllocatedUnpackBuffer, RectSize);
+        _LockedRect.pBits = GetGraphicsDisplay()->GetGpuDevice()->LockUnpackPixelBufferIndex (_AllocatedUnpackBuffer, RectSize);
         pLockedRect->pBits = ( (BYTE *) _LockedRect.pBits);
         pLockedRect->Pitch = ( ( (RectWidth * BytePerPixel + (unpack_alignment - 1) ) >> (halfUnpack) ) << (halfUnpack) );
       }
@@ -263,10 +247,10 @@ namespace nux
       int h = _Rect.bottom - _Rect.top;
       CHECKGL ( glBindTexture (_STextureTarget, _BaseTexture->_OpenGLID) );
 
-      if (GetGpuDevice()->UsePixelBufferObjects() )
+      if (GetGraphicsDisplay()->GetGpuDevice()->UsePixelBufferObjects() )
       {
         // Unmap the texture image buffer
-        GetGpuDevice()->BindUnpackPixelBufferIndex (_AllocatedUnpackBuffer);
+        GetGraphicsDisplay()->GetGpuDevice()->BindUnpackPixelBufferIndex (_AllocatedUnpackBuffer);
         CHECKGL ( glUnmapBufferARB (GL_PIXEL_UNPACK_BUFFER_ARB) );
         DataPtr = NUX_BUFFER_OFFSET (0);
       }
@@ -410,10 +394,10 @@ namespace nux
       nuxDebugMsg (TEXT("[IOpenGLSurface::UnlockRect] Incorrect Texture Target."));
     }
 
-    if (GetGpuDevice()->UsePixelBufferObjects() )
+    if (GetGraphicsDisplay()->GetGpuDevice()->UsePixelBufferObjects() )
     {
       CHECKGL ( glBindBufferARB (GL_PIXEL_UNPACK_BUFFER_ARB, 0) );
-      GetGpuDevice()->FreeUnpackPixelBufferIndex (_AllocatedUnpackBuffer);
+      GetGraphicsDisplay()->GetGpuDevice()->FreeUnpackPixelBufferIndex (_AllocatedUnpackBuffer);
     }
     else
     {
@@ -424,7 +408,7 @@ namespace nux
       }
     }
 
-    CHECKGL ( glPixelStorei (GL_UNPACK_ALIGNMENT, GetGpuDevice()->GetPixelStoreAlignment() ) );
+    CHECKGL ( glPixelStorei (GL_UNPACK_ALIGNMENT, GetGraphicsDisplay()->GetGpuDevice()->GetPixelStoreAlignment() ) );
 
     _LockedRect.pBits = 0;
     _LockedRect.Pitch = 0;
@@ -445,8 +429,7 @@ namespace nux
     nuxAssert ( texheight > 0 ); // Should never happen
     nuxAssert ( size > 0 ); // Should never happen
 
-    BYTE *DummyBuffer = new BYTE[size];
-    Memset (DummyBuffer, 0, size);
+    BYTE *DummyBuffer = (BYTE *) calloc (size, sizeof(BYTE));
 
     CHECKGL ( glPixelStorei (GL_UNPACK_ALIGNMENT, MemAlignment) );
 
@@ -519,7 +502,7 @@ namespace nux
       }
     }
 
-    delete [] DummyBuffer;
+    free (DummyBuffer);
 
 
     //    { //[DEBUGGING - Red Texture]
@@ -540,7 +523,7 @@ namespace nux
     //        delete [] color_array;
     //    }
 
-    CHECKGL (glPixelStorei (GL_UNPACK_ALIGNMENT, GetGpuDevice ()->GetPixelStoreAlignment ()));
+    CHECKGL (glPixelStorei (GL_UNPACK_ALIGNMENT, GetGraphicsDisplay()->GetGpuDevice()->GetPixelStoreAlignment()));
 
     _Initialized = true;
     return OGL_OK;
