@@ -28,17 +28,12 @@ namespace nux
   class GpuDevice;
   class GraphicsEngine;
 
-  GraphicsDisplay *GetThreadGLWindow (); // Deprecated use GetDisplayDevice
-  GraphicsDisplay *GetDisplayDevice ();
-  GpuDevice *GetThreadGLDeviceFactory (); // Deprecated use GetGpuDevice
-  GpuDevice *GetGpuDevice ();
-  GraphicsEngine *GetThreadGraphicsContext();
+  GraphicsDisplay* GetGraphicsDisplay();
 
   #define NUX_FINDRESOURCELOCATION(a) GNuxGraphicsResources.FindResourceLocation(a, false)()
   #define NUX_FIND_RESOURCE_LOCATION_NOFAIL(a) GNuxGraphicsResources.FindResourceLocation(a, true)()
 
   inlDeclareThreadLocalStorage (GraphicsDisplay *, 1, _TLS_GraphicsDisplay);
-  inlDeclareThreadLocalStorage (GpuDevice *, 2, _TLS_GpuDevice_);
 
   void NuxGraphicsInitialize();
 }
